/* newslash-test-helper - newslash test helper */
const assert = require("assert").strict;
const puptest = require("./puptest");

let page;

exports.init = function init() {
  exports.username = process.env.TEST_USER;
  exports.password = process.env.TEST_PASSWORD;
  exports.baseUrl = process.env.TEST_HOST; // "https://newslash-dev:3000";

  assert.ok(exports.username.length > 0, "username.length > 0");
  assert.ok(exports.password.length > 0, "password.length > 0");
  assert.ok(exports.baseUrl.length > 0, "baseUrl.length > 0");
         
  return puptest.newPage().then(p => {
    page = p;
    p.on('pageerror', err => {
      console.error(err);
    });
    page.on('console', msg => {
      for (let i = 0; i < msg.args().length; ++i)
        console.log(`${i}: ${msg.args()[i]}`);
    });
    return p.setViewport({width: 1024, height: 800});
  });
  
};

exports.done = function done() {
  if (page.browser) {
    return page.browser().close();
  } else {
    return new Promise((resolve, reject) => {resolve();});
  }
};

exports.getPage = function getPage() {
  return new puptest.assertPageAction(page, exports.baseUrl);
};

exports.login = async function login() {
  const p = this.getPage();
  await p.goto('/');

      // check popup shown
  await p.click('#account-menu a.toggle-login-modal');
  await p.waitForSelector('#login-modal', {timeout: 100});
  await p.$evalOk('#login-modal', el => el.classList.contains("open"));

  // check popup close
  await p.click('#login-modal form .btn-default');
  await p.waitForSelector('#login-modal', {hidden: true, timeout: 100});
  await p.$evalNotOk('#login-modal', el => el.classList.contains("open"));

  // popup open
  await p.click('#account-menu .toggle-login-modal');
  await p.waitForSelector('#login-modal', {timeout: 100});

  // input ID/pass
  await p.type('#login-nickname', this.username);
  await p.type('#login-passwd', this.password);

  // click login
  await p.clickAndWaitForNavigation('#login-modal .dialog-footer .btn-primary',
                                        {timeout: 1000});

  // check login succeeded
  const regex = new RegExp(`^\\s*${this.username}\\s*$`);
  await p.$evalMatch(regex, '#user-menu a', el => el.innerHTML.trim());
  
};

exports.logout = async function logout() {
  const p = this.getPage();
  await p.goto('/');

  // click user menu
  await p.click('#user-menu a.dropdown-toggle');
  await p.waitForSelector('#user-menu ul.dropdown-menu', {timeout: 100});

  // clock logout
  await p.clickAndWaitForNavigation('#user-menu ul.dropdown-menu a[href^="/logout"]',
                                        {timeout: 1000});

  // check login succeeded
  await p.waitForSelector('#account-menu .toggle-login-modal', {timeout: 100});
};

exports.postSubmission = async function postSubmission(postData) {
  assert.ok(postData.title && postData.title.length > 0, "postData has title");
  assert.ok(postData.body && postData.body.length > 0, "postData has body");

  const p = this.getPage();
  await p.goto('/submission/new');
  
  await p.type('#submission-editor input[name="title"]', postData.title);
  await p.type('#submission-editor textarea[name="introtext"]', postData.body);
  if (postData.url) {
    await p.type('#submission-editor input[name="url"]', postData.url);
  }
  if (postData.mail) {
    await p.type('#submission-editor input[name="email"]', postData.mail);
  }
  if (postData.tag) {
    await p.type('#submission-editor input[name="tag-single"]', postData.tag);
  }

  // click preview
  await p.click('#submission-editor .post-preview button[type="submit"]');
  await p.waitForSelector('#submission-editor .content-editor', {hidden: true, timeout: 100});
  await p.$evalEqual(postData.title, '#post-preview header h1 span', el => el.innerHTML.trim());

  // click post
  await p.click('#submission-editor .post-submit button[type="submit"]');
  await p.waitForSelector('#submission-editor .post-submit', {hidden: true, timeout: 1000});
  await p.waitForSelector('#submission-editor .post-done', {timeout: 1000});
};

