# -*-Perl-*-
# DOM and runtime script test
use Mojo::Base -strict;
use Test::More;
use Test::Mojo;
use Newslash::Util::DOMTestRunner;
use FindBin;
use Data::Dumper;

my $t = Test::Mojo->new('Newslash::Web');
my $dom_test = Newslash::Util::DOMTestRunner->new('Newslash::Web',
                                                  ignore_messages => ['adsbygoogle'],
                                                  domtest_finish_timeout => 3000,
                                                  domtest_script => "$FindBin::Bin/app.js",
                                                  mode => $t->app->mode,
                                                 );

sub test_path {
    my ($path, $code) = @_;
    $code ||= 200;
    if (!$path) {
        fail("invalid path: $path");
        return;
    }
    $t->get_ok($path)->status_is($code, "GET $path return $code");
  SKIP: {
        skip "get NG for $path", 1 if !$t->success;
        $dom_test->test($path);
    }
}

#index page
test_path("/");

# top directory
for my $path (qw[recent popular journals submissions polls]) { #comments
    test_path("/$path");
}

# story
my $items = $t->app->model('stories')->select(limit => 1);
ok($items->[0]->{sid}, "retrive valid sid");
my $path = "/story/$items->[0]->{sid}";
test_path($path);

# comment
$items = $t->app->model('comments')->select(limit => 1);
ok($items->[0]->{id}, "retrive valid cid");
$path = "/comment/$items->[0]->{id}";
test_path($path);

# journal
$items = $t->app->model('journals')->select(limit => 1);
ok($items->[0]->{id}, "retrive valid journal_id");
$path = "/journal/$items->[0]->{id}";
test_path($path, 301);

my $user = $t->app->users->get_user(uid => $items->[0]->{uid});
ok($user, "get valid user");
$path = "/~$user->{nickname}/journal/$items->[0]->{id}";
test_path($path, 200);

# submission
$items = $t->app->model('submissions')->select(limit => 1, order_by => { subid => 'DESC'});
#diag Dumper $items;
ok($items->[0]->{subid}, "retrive valid subid");
$path = "/submission/$items->[0]->{subid}";
test_path($path);

# polls
$items = $t->app->model('polls')->select(limit => 1);
ok($items->[0]->{qid}, "retrive valid polls qid");
$path = "/poll/$items->[0]->{qid}";
test_path($path);

# archive

# tag
SKIP: {
    skip "mode is 'test'", 4 if ($t->app->mode eq 'test');
    test_path("/tag/");
    test_path("/tag/linux/");
    test_path("/tag/linux/2");
    test_path("/tag/linux/story/");
    test_path("/tag/linux/story/2/");
}

# user


done_testing;

