package Newslash::Eventd::Daemon;
use Mojo::Base -base;

use Mojo::Server::Daemon;
use Mojo::Util qw(dumper);

use Newslash::Eventd::Handlers;
my $polling_interval = 3; # seconds

sub fetch_event_que {
    my $daemon = shift;
    my $app = shift;
    my $handlers = shift;

    $app->log->debug('fetch event queue...');

    my $events = $app->model('events');
    my $rs = eval { $events->fetch(10) };
    if ($@) {
        $app->log->error("Eventd::Daemon::fetch_event_que: fetch event error. message(s):");
        $app->log->error($@);
    }
    if (!$rs) {
        $app->log->fatal("event fetch failed!");
    }

    if (!$rs || !@$rs) {
        $daemon->ioloop->timer($polling_interval => sub {$daemon->emit("fetch_event_que")});
        $app->log->debug("$polling_interval sec wait, then run next fetch");
        return;
    }

    for my $item (@$rs) {
        $app->log->debug("event received");
        eval { $handlers->execute($item) };
        # check exception
        if ($@) {
            $app->log->error("Eventd::Daemon::fetch_event_que: handler execution error. message(s):");
            $app->log->error($@);
        }
    }

    $daemon->ioloop->timer(0.1 => sub {$daemon->emit("fetch_event_que")});
    $app->log->debug("run next fetch");

    return;
}

sub start {
    my $class = shift;
    my $options = shift || {};
    my $daemon = Mojo::Server::Daemon->new;
    my $app = $daemon->build_app('Newslash::Eventd');
    my $listen = [];

    if ($ENV{'MOJO_LISTEN'}) {
        push @$listen, $ENV{'MOJO_LISTEN'};
    }
    else {
        push @$listen, 'http://*:8888';
    }

    $daemon->listen($listen);

    my $handlers = Newslash::Eventd::Handlers->new($app, $options);
    $daemon->on(fetch_event_que => sub {
                    fetch_event_que($daemon, $app, $handlers);
                });

    $daemon->ioloop->delay(sub {$daemon->emit("fetch_event_que")});
    $daemon->run;
}

