package Newslash::Eventd::Handler;
use Mojo::Base 'Mojo::EventEmitter';
use File::Spec;
use Mojo::Util qw(dumper);
use Data::Dumper;

sub new {
    my ($class, $app, $options) = @_;
    $options ||= {};
    bless {
           _options => $options,
           _app => $app,
           _error => "",
           _errorno => 0,
           template_paths => [
                              File::Spec->join($app->home, 'templates'),
                             ],
           _test => $options->{test} || 0,
          }, $class;
}

sub app {
    return shift->{_app};
}

sub options {
    return shift->{_options};
}

sub inactivate_event {
    my ($self, $event_id, $result) = @_;

    if (!defined $result) {
        $self->app->log->error("event cannot unactivate: no result given. id: $event_id");
        return;
    }

    my $rs = $self->app->model('events')->update(id => $event_id, active => 0, result => $result);
    if (!$rs) {
        $self->app->log->error("event cannot unactivate. id: $event_id");
    }
    return $rs;
}

sub render {
    my ($self, $template_name, $vars, $opts) = @_;
    $opts ||= {};

    my $renderer = $self->app->tt2renderer;
    my $rs = $renderer->render($template_name, $vars);
    return $rs;
}

sub send_email {
    my $self = shift;
    my $args = {@_};

    if ($self->{_test}) {
        $self->{_test_mailque} //= [];
        my $result = $self->app->sendmail(render_only => 1, @_);
        $result ||= {};
        push @{$self->{_test_mailque}}, $result;
    }
    return $self->app->sendmail(@_);
}

sub create_message {
    my $self = shift;
    my $params = {@_};

    my $uid = $params->{to};
    my $from_uid = $params->{from};
    my $template = $params->{template};
    my $vars = $params->{vars} || {};
    my $code = $params->{code};
    my $type = $params->{type} || "";

    return if (!$uid || !$template || !$code);
    my $messages = $self->app->model('messages');

    my $render_vars = {%$vars};
    my $users = $self->app->model("users");
    my $from_user = $users->select(uid => $from_uid) || {};
    my $to_user = $users->select(uid => $uid) || {};
    $render_vars->{from_user} = $from_user;
    $render_vars->{to_user} = $to_user;

    # get message mode
    my $message_configs = $users->messages->select(uid => $uid);
    my $mode = -1;
    for my $conf (@$message_configs) {
        if ($conf->{code} == $code) {
            $mode = $conf->{mode};
            last;
        }
    }
    return 0 if ($mode == -1);

    # render title and body
    $render_vars->{format} = "text/html";

    $render_vars->{target} = "title";
    my $title = eval { $self->render($template, $render_vars); };
    $self->app->log->error("messages: no title; template: $template; error: $@") if !$title;

    $render_vars->{target} = "body";
    my $body = eval { $self->render($template, $render_vars); };
    $self->app->log->error("messages: no body; template: $template; error: $@") if !$body;

    my $rs = $messages->create(to => $uid,
                               from => $from_uid,
                               title => $title,
                               body => $body,
                               code => $code,
                               type => $type,
                              );
    return $rs;
}

1;
