package Newslash::Eventd::Handler::Comment;
use Mojo::Base 'Newslash::Eventd::Handler';

sub post {
    my ($self, $item) = @_;

    my $comment_id = $item->{related_id};
    my $uid = $item->{source_user_id};
    my $events = $self->app->model('events');
    my $messages = $self->app->model('messages');

    $self->app->log->debug("event comment post processing... id: $comment_id, uid: $uid");

    # first, send reply notify
    my $comments = $self->app->model('comments');
    my $comment = $comments->select(cid => $comment_id);
    if (!$comment) {
        $self->app->log->warn("invalid comment. id: $comment_id");
        $self->inactivate_event($item->{event_id}, -2);
        return;
    }

    my $users = $self->app->model('users');

    if ($comment->{pid}) {
        my $parent = $comments->select(cid => $comment->{pid});
        if (!$parent) {
            $self->app->log->warn("invalid parent comment. id: $comment->{pid}, cid: $comment_id");
            $self->inactivate_event($item->{event_id}, -2);
            return;
        }
        if (!$users->is_anonymous_uid($parent->{uid})) {
            #my $parent_author = $users->select(uid => $parent->{uid});
            my $vars = { comment => $comment,
                         parent => $parent };

            my $rs = $self->create_message(to => $parent->{uid},
                                           from => $uid,
                                           template => "message/comment_reply",
                                           vars => $vars,
                                           code => 4, # comment reply
                                           type => "notify");
            if ($rs) {
                $self->app->log->debug("message created. id: $item->{event_id}, message_id: $rs");
            }
            else {
                $self->app->log->debug("message cannot create. id: $item->{event_id}, to: $parent->{uid}");
            }
        }
    }

    # send journal comment notify
    my $discussion = $self->app->model('discussions')->select(discussion_id => $comment->{sid});
    if (!$discussion) {
        $self->app->log->warn("invalid discussion. cid: $comment_id");
        $self->inactivate_event($item->{event_id}, -2);
        return;
    }
    if ($discussion->{dkid} == 3) {
        my $journal = $self->app->model('journals')->select(discussion_id => $discussion->{id});
        if (!$journal) {
            $self->app->log->warn("invalid journal. discussion_id: $discussion->{id}, cid: $comment_id");
            $self->inactivate_event($item->{event_id}, -2);
            return;
        }
        my $vars = { comment => $comment,
                     journal => $journal };
        my $rs = $self->create_message(to => $journal->{uid},
                                       from => $uid,
                                       template => "message/journal_reply",
                                       vars => $vars,
                                       code => 7, # journal reply
                                       type => "notify");
        if ($rs) {
            $self->app->log->debug("message created. id: $item->{event_id}, message_id: $rs");
        }
        else {
            $self->app->log->debug("message cannot create. id: $item->{event_id}, to: $journal->{uid}");
        }
    }

    $self->inactivate_event($item->{event_id}, 0);
    return 1;
}

1;
