package Newslash::Eventd::Handlers;
use Mojo::Base 'Newslash::Eventd::Handler';

use File::Spec;
#use Mojo::Loader qw(load_class);
use Mojo::Util qw(dumper class_to_path);

has 'last_error';

sub load_class { # TODO: mojolicious 5.x has no load_class(),  but load().
    my ($module) = @_;

    # Check module name
    return 1 if !$module || $module !~ /^\w(?:[\w:']*\w)?$/;

    # Load
    return undef if $module->can('new') || eval "require $module; 1";

    # Exists
    return 1 if $@ =~ /^Can't locate \Q@{[class_to_path $module]}\E in \@INC/;

    # Real error
    return Mojo::Exception->new($@);
}


sub new {
    my ($class, $app, $options) = @_;
    bless {
           _options => $options,
           _app => $app,
           _error => "",
           _errorno => 0,
           template_paths => [
                              File::Spec->join($app->home, 'templates'),
                             ],
           _namespaces => $options->{handlers_namespaces} || ['Newslash::Eventd::Handler',],
           _handlers => {},
          }, $class;
}

sub _load_handler {
    my ($self, $event_type) = @_;

    my $camelized = Mojo::Util::camelize($event_type);
    my $class_name = "";

    my @errors;
    for my $ns (@{$self->{_namespaces}}) {
        $class_name = "$ns::$camelized";
        if (my $e = load_class($class_name)) { 
            # load failed
            if (ref($e)) {
                # execption occured
                push @errors, $e;
                $class_name = "";
                next;
            }
            else {
                push @errors, "Not Found: $class_name";
                $class_name = "";
                next;
            }
        }
        last;
    }
    if (!$class_name) {
        $self->app->log->error("handler load failed: " . join("\n", @errors));
        return;
    }
    my $handler = $class_name->new($self->app, $self->options);
    return $handler;
}

sub execute {
    my ($self, $item) = @_;

    return if !$item->{active};

    my $handler = $self->{_handlers}->{$item->{event_type}};
    my @execptions;

    if (!$handler) {
        $handler = $self->_load_handler($item->{event_type});
        if (!$handler) {
            # no handler found
            $self->app->log->debug("no handler found for '$item->{event_type}'");
            $self->inactivate_event($item->{event_id}, -128);
            $self->last_error("NO_HANDLER_FOUND");
            return;
        }
        $self->{_handlers}->{$item->{event_type}}= $handler;
    }

    my $f = $handler->can($item->{event_subtype});
    if (!$f) {
        # no handler found
        $self->app->log->debug("no '$item->{event_subtype}' handler found for '$item->{event_type}'");
        $self->inactivate_event($item->{event_id}, -127);
        $self->last_error("NO_EVENT_FOUND");
        return;
    }

    $self->app->log->debug("handler '$item->{event_subtype}' found for '$item->{event_type}'");
    return $f->($handler, $item);
}



1;
