package Newslash::Model::Topics;
use Newslash::Model::Base -base;

use Data::Dumper;

sub select {
    my $self = shift;
    my $params = {@_};

    my $rs = $self->generic_select(table => 'topics',
                                   unique_keys => [qw(tid keyword textname)],
                                   keys => [qw(series submittable searchable storypickabe)],
                                   params => $params);
    return $rs;
}

sub get_topic_nexus {
    my $self = shift;
    my $rs = $self->generic_select(table => 'topic_nexus',
                                   unique_keys => [qw(tid)],
                                   keys => [qw(tid current_qid)]);
    return $rs;
}

sub _select_topic_parents {
    my $self = shift;
    my $params = {@_};

    my $rs = $self->generic_select(table => 'topic_parents',
                                   keys => [qw(tid)],
                                   params => $params);
    return $rs;
}

sub get_primary_topic_from_tags {
    my ($self, $tags) = @_;
    return if !$tags || !@$tags;

    my $dbh = $self->connect_db;

    my @placeholders = map { "?" } @$tags;
    my $placeholder = join(", ", @placeholders);

    my $sql = <<"EOSQL";
SELECT * from topics
  WHERE keyword IN ($placeholder)
        OR textname IN ($placeholder)
EOSQL

    my $sth = $dbh->prepare($sql);
    $sth->execute(@$tags, @$tags);
    my $rs = $sth->fetchall_arrayref({});
    return if !$rs;

    my $index = {};
    for my $item (@$rs) {
        $index->{$item->{keyword}} = $item;
        $index->{$item->{textname}} = $item;
    }

    # scan topics by given order
    my $topic;
    for my $tag (@$tags) {
        next if !defined $index->{$tag};
        next if $index->{$tag}->{submittable} ne 'yes';
        $topic = $index->{$tag};
        last;
    }

    return $topic;
}


1;
