package Newslash::Plugin::ADRenderer;
use Mojo::Base 'Mojolicious::Plugin';

use strict;
use warnings;
use utf8;
use feature ':5.10';
use Data::Dumper;

sub register {
  my ($self, $app, $conf) = @_;
  $self->{app} = $app;
  $app->helper(ad_renderer => sub { state $ad_code = $self; });
  $app->helper(ad_code => sub { get_code(@_); });
}

sub get_code {
    my ($c, $ad_name, $user) = @_;
    return if !$ad_name;
    if (!$user) {
        $user = $c->stash('user');
    }
    return if !$user;

    my $cache_param = { no_cache => ($user && $user->{is_admin} && $user->{config}->{system}->{disable_cache}), };
    my $ad_code = $c->ccache->model('ad_codes', $cache_param)->select(name => $ad_name);

    if (!$ad_code) {
        return;
    }
    return $ad_code->{content};
}


1;

=encoding utf8

=head1 NAME

Newslash::Plugin::ViewFunctions - Utility functions for MVC's View

=head1 SYNOPSIS

  # Mojolicious
  $app->plugin('Newslash::Plugin::ViewFunctions');

=head1 DESCRIPTION

L<Newslash::Plugin::ViewFunctions> porvides some utility functions to use
in View templates.

=head1 METHODS

=head2 register

  $plugin->register(Mojolicious->new);

Register functions in L<Mojolicious> application.

=head2 format_timestamp

  $plugin->format_timestamp(user => $user, epoch => $epoch, format => "user")
  $plugin->format_timestamp(datetime => $dt, format => "simple")

Return formated string.

=head1 SEE ALSO

L<Mojolicious>, L<Mojolicious::Guides>, L<http://mojolicious.org>.

=cut
