package Newslash::Plugin::RendererHelper;
use Mojo::Base 'Mojolicious::Plugin';
use Time::HiRes qw(time);

sub register {
    my ($self, $app, $conf) = @_;
    $self->{app} = $app;
    $self->{conf} = $conf;
    $self->{functions} = {};

    $app->helper(renderer_helper => sub { state $renderer_helper = $self; });
}

sub add_function {
    my ($self, $namespace, $name, $function) = @_;

    $self->{functions} ||= {};
    $self->{functions}->{$namespace} ||= {};

    $self->{functions}->{$namespace}->{$name} = $function;
}

sub get_functions {
    my ($self, $namespace) = @_;
    return if !$self->{functions};
    return $self->{functions}->{$namespace} || {};
}

1;

=encoding utf8

=head1 NAME

Newslash::Plugin::RendererHelper - Helper functions for Renderer

=head1 SYNOPSIS

  # Mojolicious
  $app->plugin('Newslash::Plugin::RendererHelper');

=head1 DESCRIPTION

L<Newslash::Plugin::RendererHelper> porvides some function to help renderer.


=head1 METHODS

=head2 register

  $plugin->register(Mojolicious->new);

Register epoch in L<Mojolicious> application.

=head1 SEE ALSO

L<Mojolicious>, L<Mojolicious::Guides>, L<http://mojolicious.org>.

=cut
