package Newslash::StatsAggregator;
use Mojo::Base qw(Newslash::Scripts);

use Data::Dumper;

use constant CONFIG_FILE => '/etc/newslash/newslash.conf';

sub startup {
    my $self = shift;
    my $app = $self->app;

    # set log level
    $app->log->level("warn");

    # load config file
    if ($app->mode eq 'production' && -e CONFIG_FILE) {
        $app->plugin('Newslash::Plugin::YAMLConfig', file => CONFIG_FILE);
    }

    # use TimeLimitedCache ($app->cache)
    $app->plugin('Newslash::Plugin::TimeLimitedCache');

    # use KeyValue Store ($app->kvs)
    $app->plugin('Newslash::Plugin::KeyValueStore');

    # add Model Loader
    $app->plugin('Newslash::Plugin::Model');

    # Stats Logger
    $app->plugin('Newslash::Plugin::Stats');

    # set log file
    $app->log->level("debug");

    my $log_base_dir = ($app->config("Log") || {})->{base_dir} || ".";
    my $log_pathname = "$log_base_dir/stats_aggregator.log";

    if (-d $log_base_dir &&
        -w $log_pathname || -w $log_base_dir) {
        $app->log->info("logs will be outputed to $log_pathname...");
        $app->log(Mojo::Log->new(level => "debug", path => $log_pathname));
    }

    # set log level

}

sub run {
    my $self = shift;
    my $verbose = 2;

    $self->app->stats->aggregate_story_pageview_counters;

    return 0;
}

1;
