package Newslash::Web::Controller::API::Admin::Sidebar;
use Mojo::Base 'Mojolicious::Controller';
use Mojo::Util qw(dumper);

sub get {
    my $c = shift;
    my $data = $c->req->params->to_hash || {};

    if ($data->{id}) {
        my $boxes = $c->app->model('boxes');
        my $item = $boxes->select(id => $data->{id});
        if (!$item) {
            $c->rendered(404);
            $c->render(json => {error => 1, message => "not_found"});
            return;
        }
        return $c->render(json => {item => $item});
    }

    return $c->_list(@_);
}

sub _list {
    my $c = shift;
    my $boxes = $c->app->model('boxes');
    my $items = $boxes->select;

    if (!defined $items) {
        $c->rendered(500);
        $c->render(json => {error => 1, message => $boxes->last_error});
        return;
    }

    $c->render(json => {items => $items});
    return;
}

sub post {
    my $c = shift;
    my $data = $c->req->json || {};
    my $action = $data->{action} || "";

    if ($data->{action} eq "create") {
        return $c->_create(@_);
    }
    elsif ($data->{action} eq "update") {
        return $c->_update(@_);
    }
    elsif ($data->{action} eq "delete") {
        return $c->_delete(@_);
    }
    $c->rendered(400);
    $c->render(json => {error => 1, message => "invalid_request"});
    return;
}

sub _create {
    my $c = shift;
    my $boxes = $c->app->model('boxes');
    my $data = $c->req->json;

    my $params = {};
    my $message = "";
    $message = "no_name_given" if !$data->{name};

    if (length $message) {
        $c->rendered(400);
        $c->render(json => {error => 1, message => $message});
        return;
    }

    for my $i (qw(name security_level title comment type model query_params limit template)) {
        $params->{$i} = $data->{$i} if defined $data->{$i};
    }


    my $id = $boxes->create(%$params);
    if (!$id) {
        $c->rendered(400);
        $c->render(json => {error => 1, message => $boxes->last_error});
    }
    $c->render(json => {id => $id});
    return;

}

sub _update {
    my $c = shift;
    my $boxes = $c->app->model('boxes');
    my $data = $c->req->json;

    my $params = {};
    my $message = "";
    $message = "no_id_given" if !$data->{id};

    if (length $message) {
        $c->rendered(400);
        $c->render(json => {error => 1, message => $message});
        return;
    }

    for my $i (qw(id name security_level title comment type model query_params limit template)) {
        $params->{$i} = $data->{$i} if defined $data->{$i};
    }


    my $id = $boxes->update(%$params);
    if (!$id) {
        $c->rendered(400);
        $c->render(json => {error => 1, message => $boxes->last_error});
    }
    $c->render(json => {id => $id});
    return;
}

sub _delete {
    my $c = shift;
    my $boxes = $c->app->model('boxes');
    my $data = $c->req->json || {};

    my $message = "";
    $message = "no_id_given" if !$data->{id};

    if (length $message) {
        $c->render(json => {error => 1, message => $message});
        return;
    }

    my $rs = $boxes->delete(id => $data->{id});

    if (!$rs) {
        $c->render(json => {error => 1, message => $boxes->last_error});
        return;
    }
    $c->render(json => {error => 0, id => $data->{id}});
    return;
}


1;

