package Newslash::Web::Controller::API::Login;
use Mojo::Base 'Mojolicious::Controller';
use Mojo::Util qw(dumper);
use Mojo::JSON qw(decode_json encode_json to_json from_json);



sub login {
    my $c = shift;
    my $params = $c->req->json;
    my $nickname = $params->{nickname};
    my $password = $params->{password};
    my $type = $params->{type} || "login";

    if ($type eq "reset_password") {
        return $c->_reset_password;
    }

    if (defined $nickname && defined $password) {
        my $users = $c->model('users');
        my $user = $users->authentification($nickname, $password);
        if ($user) {
            if (!$c->user_auth->do_login($user, $c)) {
                $c->rendered(500);
                return;
            }
            $c->render(json => { nickname => $nickname });
            $c->event_que->emit("user", "login", $user->{uid});
            $c->stats->add_event_counter("user login");
            return;
        }
    }

    $c->render(json => { error => 1,
                         nickname => $nickname,
                         reason => "auth_failed",
                         message => "invalid nickname or password" });
    $c->rendered(401);
    return;
}

sub _reset_password {
    my $c = shift;
    my $params = $c->req->json;

    if ($params->{email}) {
        my $users = $c->model('users');
        my $user = $users->select(realemail => $params->{email});
        if (!$user) {
            # mask invalid_user error
            $c->app->log->info("API::Login: invalid reset password request for $params->{email}.");
            $c->render(json => {});
            return;
        }
        my $rs = $c->users->reset_password($user);
        if (!$rs) {
            $c->render(json => { error => $c->users->last_error || "SERVER_ERROR" });
            $c->rendered(400);
            return;
        }
        $c->render(json => {});
        return;
    }
    $c->render(json => { error => 1, message => "INVALID_EMAIL" });
    $c->rendered(400);
    return;
}


1;
