package Newslash::Web::Controller::API::User;
use Mojo::Base 'Mojolicious::Controller';
use Mojo::Util qw(dumper);
use Mojo::JSON qw(decode_json encode_json to_json from_json);
use Email::Valid;

=pod

get - get user information and data, settins.

=cut

sub get {
    my $c = shift;
    my $params = $c->req->query_params->to_hash;
    my $users = $c->model('users');
    my $user = $c->stash('user');

    my $type = $params->{type} || "";
    my $uid = $params->{user_id} || $user->{uid};
    if ($uid != $user->{uid} && !$user->{is_admin}) {
        $c->render(json => { error => 1, message => "no_privileges" });
        $c->rendered(403);
        return;
    }

    if ($type eq 'config') {
        my $rs = $users->configs->select(uid => $uid);
        if ($rs) {
            $c->render(json => { config => $rs });
            return;
        }
    }
    elsif ($type eq 'message_config') {
        my $rs = $users->messages->select(uid => $uid);
        if ($rs) {
            $c->render(json => { config => $rs });
            return;
        }
    }
    elsif ($type eq 'sidebar') {
        my $rs = $users->sidebar->select(uid => $uid);
        if ($rs && @$rs) {
            $c->render(json => { items => $rs });
            return;
        }
        # fallback
        $rs = $users->sidebar->select(uid => $users->anonymous_user->{uid});
        if ($rs) {
            $c->render(json => { items => $rs });
            return;
        }
        $c->render(json => { items => [] });
        return;
    }
    elsif ($type eq 'all') {
        if (!$user->{is_admin}) {
            $c->render(json => { error => 1, message => "no_privileges" });
            $c->rendered(403);
            return;
        }
        my $networks = $c->model('networks');
        my $user;
        my $activities;
        if ($params->{srcid}) {
            $user = { is_network => 1, };
            $activities = $networks->activities->select(srcid => $params->{srcid});
            my $acl2 = $users->acl2->select(srcid => $params->{srcid});
            my $acl2_log = $users->acl2->select_log(srcid => $params->{srcid});
            my @values = map { $_->{al2tid} } @{$acl2->{values}};
            $acl2->{values} = \@values;
            $user->{acl2} = $acl2 || {};
            $user->{acl2_log} = $acl2_log || [];
            $user->{activities} = $activities;
            $c->render(json => { item => $user });
            return;
        }

        if ($params->{uid}) {
            $user = $users->select(uid => $params->{uid});
            $activities = $networks->activities->select(uid => $params->{uid});
        }
        elsif ($params->{nickname}) {
            $user = $users->select(nickname => $params->{nickname});
            if ($user) {
                $activities = $networks->activities->select(uid => $uid);
            }
        }
        if ($user && $uid) {
            my $acl2 = $users->acl2->select(uid => $uid);
            my $acl2_log = $users->acl2->select_log(uid => $uid);
            my @values = map { $_->{al2tid} } @{$acl2->{values}};
            $acl2->{values} = \@values;
            $user->{acl2} = $acl2 || {};
            $user->{acl2_log} = $acl2_log || [];
            $user->{activities} = $activities;
            $c->render(json => { item => $user });
            return;
        }
        $c->render(json => { error => 1, message => "not_found" });
        $c->rendered(404);
        return;
    }
    else {
        my $rs;
        if ($params->{uid}) {
            $rs = $users->select(uid => $params->{uid});
        }
        elsif ($params->{nickname}) {
            $rs = $users->select(nickname => $params->{nickname});
        }

        if ($rs) {
            $c->render(json => { item => $rs });
            return;
        }
        $c->render(json => { error => 1, message => "not_found" });
        $c->rendered(404);
        return;
    }
    $c->rendered(400);
    return;
}

sub post {
    my $c = shift;
    my $user = $c->stash('user');
    my $users = $c->model('users');
    my $data = $c->req->json;

    my $uid = $data->{user_id} || $user->{uid};
    if ($uid != $user->{uid} && !$user->{is_admin}) {
        $c->render(json => { error => 1, message => "no_privileges" });
        $c->rendered(403);
        return;
    }

    if (!$user->{is_login}) {
        $c->render(json => { error => 1, message => "you are not a login user." });
        $c->rendered(400);
        return;
    }

    if ($data->{type} eq "config") {
        return $c->update_config($uid, $data);
    }

    if ($data->{type} eq "sidebar") {
        return $c->update_sidebar($uid, $data);
    }

    if ($data->{type} eq "message") {
        return $c->update_message($uid, $data);
    }

    if ($data->{type} eq "acl2") {
        return $c->update_acl2($uid, $data);
    }

    if ($data->{type} eq "email") {
        return $c->update_email($uid, $data);
    }

    if ($data->{type} eq "password") {
        return $c->update_password($uid, $data);
    }

    $c->render(json => { error => 1, message => "invalid_request" });
    $c->rendered(400);
    return;
}

sub update_config {
    my ($c, $uid, $data) = @_;
    my $users = $c->model('users');

    my $config = $data->{config} || {};
    delete $config->{id} if defined $config->{id};
    delete $config->{uid} if defined $config->{uid};
    delete $config->{user_id} if defined $config->{user_id};

    my $new_cfg = $users->configs->update(id => $uid, %$config);
    if ($new_cfg) {
        my $target_user = $users->select(uid => $uid);

        # update session cache
        $c->user_auth->update_user_cache($target_user);

        $c->render(json => {config => $new_cfg, message => ""});
        $c->event_que->emit("user", "update", $uid);
        return;
    }
    $c->render(json => { error => 1, message => $users->last_error });
    $c->rendered(400);
    return;
}

sub update_sidebar {
    my ($c, $uid, $data) = @_;
    my $users = $c->model('users');

    my $new_cfg = $users->sidebar->update(uid => $uid,
                                          config => $data->{items});
    if ($new_cfg) {
        my $target_user = $users->select(uid => $uid);

        # update session cache
        $c->user_auth->update_user_cache($target_user);

        $c->render(json => {items => $new_cfg, message => ""});
        $c->event_que->emit("user", "update", $uid);
        return;
    }
    $c->render(json => { error => 1, message => $users->last_error });
    $c->rendered(500);
    return;
}

sub update_message {
    my ($c, $uid, $data) = @_;
    my $users = $c->model('users');

    my $items = $data->{items};
    my $error = "";
    my $updated = 0;
    my $messages = $users->messages;
    for my $item (@$items) {
        my $rs = $messages->update(uid => $uid,
                                       code => $item->{code},
                                       mode => $item->{mode});
        if (defined $rs) {
            $updated += $rs;
        }
        else {
            $error = $messages->last_error;
        }
    }

    if ($updated) {
        $c->event_que->emit("user", "update_message_settings", $uid);
    }

    if (!$error) {
        $c->render(json => {items => $items, message => $error});
        return;
    }

    $c->render(json => { error => 1, message => $users->last_error });
    $c->rendered(400);
    return;
}

sub update_acl2 {
    my ($c, $uid, $data) = @_;
    my $user = $c->stash('user');
    my $users = $c->model('users');

    if (!$user->{is_admin}) {
        $c->render(json => { error => 1, message => "no_privileges" });
        $c->rendered(403);
        return;
    }
    my $rs = $users->acl2->update(uid => $uid,
                                  adin_uid => $user->{uid},
                                  types => $data->{values},
                                  comment => $data->{comment});
    if ($rs) {
        $c->user_auth->update_user_cache($user);
        $c->render(json => {item => $data});
        #$c->event_que->emit("user", "update_acl2", $uid);
        return;
    }
    $c->render(json => { error => 1, message => $users->last_error });
    $c->rendered(500);
    return;
}

sub update_email {
    my ($c, $uid, $data) = @_;
    my $user = $c->stash('user');
    my $users = $c->model('users');

    my $address = $data->{address};
    my $message = "";

    if (!Email::Valid->address($address)) {
        $message = "INVALID_ADDRESS";
    }
    else {
        my $rs = $c->users->request_change_email($user, $address);
        if (!$rs) {
            $message = $c->users->last_error;
        }
    }

    if (!$message) {
        $c->render(json => { uid => $user->{uid}, email => $address });
        return;
    }

    $c->render(json => { error => 1, message => $message });
    $c->rendered(400);
    return;

}

sub update_password {
    my ($c, $uid, $data) = @_;
    my $user = $c->stash('user');
    my $users = $c->model('users');


    if (!$data->{current_password} || !$data->{new_password}) {
        $c->render(json => { error => 1, message => "no_password_given" });
        $c->rendered(400);
        return;
    }

    my $rs = $c->users->update_password($user,
                                        $data->{current_password},
                                        $data->{new_password});
    if ($rs) {
        $c->render(json => { uid => $user->{uid} });
        return;
    }
    # error
    $c->render(json => { error => 1, message => $c->users->last_error });
    $c->rendered(400);
    return;

};

sub newuser_password {
    my $c = shift;
    my $params = $c->req->json;

    if ($params->{token}) {
        # token mode
        if(!$c->users->update_password_by_token($params->{nickname},
                                                $params->{token},
                                                $params->{password})) {
            $c->render(json => { error => $c->users->last_error || 1 });
            $c->rendered(400);
            return;
        }
        $c->render(json => {});
        return;
    }
    $c->render(json => { error => 1, message => "invalid_request" });
    $c->rendered(400);
    return;
}

sub validate_new_user {
    my $c = shift;
    my $params = $c->req->json;
    my $nickname = $params->{'nickname'};
    my $email = $params->{'email'};
    $email =~ s/\s//g; # strip whitespace

    my ($id_error, $email_error) = (0, 0);
    $id_error = $c->users->validate_nickname($nickname) ? 0 : $c->users->last_error;
    $email_error = $c->users->validate_email($email) ? 0 : $c->users->last_error;

    if ($id_error || $email_error) {
        $c->render(json => { error => 1,
                             id_error => $id_error || "",
                             email_error => $email_error || "",
                             nickname => $nickname,
                             email => $email });
        $c->rendered(400);
        return;
    }

    $c->render(json => { nickname => $nickname,
                         email => $email });
}

sub create_new_user {
    my $c = shift;
    my $params = $c->req->json;
    my $nickname = $params->{nickname};
    my $email = $params->{email};
    my $options = $params->{options} || {};

    if (ref($options) ne "HASH") {
        $c->render(json => { error => 1,
                             _message => "options parameter must be dictionary",
                             nickname => $nickname,
                             email => $email });
        $c->rendered(400);
        return;
    }

    $email =~ s/\s//g; # strip whitespace

    my $uid = $c->users->create_new_user($nickname, $email, $options);
    if(!$uid) {
        my $error = $c->users->last_error;
        $c->render(json => { error => 1,
                             id_error => $error->{id_error} || "",
                             email_error => $error->{email_error} || "",
                             nickname => $nickname,
                             email => $email });
        $c->rendered(400);
        return;
    }

    $c->render(json => { nickname => $nickname,
                         email => $email,
                         uid => $uid
                       });
    return;
}

1;
