package Newslash::Web::Controller::Rss;
use Mojo::Base 'Newslash::Web::Controller';
use Mojo::Util qw(dumper);
use List::Util qw(any);

use DateTime;
use DateTime::Format::MySQL;
use Newslash::Util::Items;

use constant SUB_SECTIONS => qw(
                                   apple askslashdot developers hardware idle interview
                                   it linux mobile opensource review science security slash yro
                              );
sub stories {
    my $c = shift;
    my $type = $c->stash("rss_type") || "";
    my $config = $c->app->config->{Rss} || {};
    my $limit = $config->{number_of_items} || 10;

    my $site_config = $c->app->config->{Site} || {};
    my $items;
    my $subtitle = "";
    my $link = "";
    my $date = $c->format_timestamp(now => "iso8601");

    if ($type eq "sradjp") {
        my $stories = $c->ccache->model('stories');
        $items = $stories->select(hide_future => 1,
                                  public_only => 1,
                                  limit => $limit,
                                  order_by => {create_time => 'desc'},
                                 );
        $link = $site_config->{base_url} . "/";
    } elsif (any { $type eq $_} (SUB_SECTIONS)) {
        # get tid
        my $topic = $c->model("topics")->select(keyword => $type);
        my $stories = $c->ccache->model('stories');
        $items = $stories->select(hide_future => 1,
                                  public_only => 1,
                                  limit => $limit,
                                  tid => $topic->{tid},
                                  order_by => {create_time => 'desc'},
                                 );
        $subtitle = "：$topic->{textname}";
        $link = $site_config->{base_url} . "/tag/$topic->{keyword}/";
    } else {
        # select by tag
        $c->rendered(404);
        return;
    }

    $c->render("rss/stories",
               items => $items,
               title => $site_config->{name} . $subtitle,
               link => $link,
               date => $date,
               rss_url => $site_config->{base_url} . $c->url_for,
              );
}

sub journals {
    my $c = shift;
    my $config = $c->app->config->{Rss} || {};
    my $limit = $config->{number_of_items} || 10;

    my $site_config = $c->app->config->{Site} || {};

    my $subtitle = "：みんなの日記";
    my $date = $c->format_timestamp(now => "iso8601");

    my $journals = $c->ccache->model('journals');
    my $items = $journals->select(limit => $limit,
                               order_by => {create_time => 'desc'},
                             );
    my $link = $site_config->{base_url} . "/journals";

    $c->render("rss/journals",
               format => "xml",
               items => $items,
               title => $site_config->{name} . $subtitle,
               link => $link,
               date => $date,
               rss_url => $site_config->{base_url} . $c->url_for,
              );
}


1;

