package Newslash::Web::Controller::Search;
use Mojo::Base 'Mojolicious::Controller';
use Mojo::UserAgent;

sub search {
    my $c = shift;
    my $user = $c->stash('user');
    my $config = $c->config->{Search} || {};
    my $host = $config->{searchd_host};
    my $port = $config->{searchd_port};
    my $scheme = "http";

    my $query = $c->param('q');
    my $start = $c->param('start') || 0;
    my $target = $c->param('target') || "story";
    my $sort_reverse = $c->param('sort_reverse') // 1;

    my $con_timeout = $config->{connect_timeout} // 10;
    my $req_timeout = $config->{request_timeout} // 10;

    # $item_per_page must be number, otherwise, searchd returns error.
    my $item_per_page = ($config->{item_per_page} || 10) + 0;

    if (!$query) {
        # no query, show search top page
        $c->render();
        return;
    }

    # query to searchd
    my $url = "$scheme://$host:$port/";
    # Non-blocking request
    my $ua  = Mojo::UserAgent->new;
    $ua->connect_timeout($con_timeout);
    $ua->request_timeout($req_timeout);
    $c->render_later;

    my $req = { query => $query,
                limit => $item_per_page,
                offset => $start,
                target => $target,
                sort => "create_time",
                sort_reverse => $sort_reverse,
             };
    $ua->post($url => {Accept => 'application/json'} => json => $req => sub {
                  my ($ua_, $tx) = @_;
                  my $message = "";

                 # $ua must exist when this callback is called,
                 # so capture it
                 if (!defined $ua) {
                     $message = "some_error_occured";
                     $c->render(query_words => $query,
                                target => $target,
                                sort_reverse => $sort_reverse,
                                message => $message,
                                error => 1,
                                status => 500,
                               );
                     $c->rendered(500);
                     return;
                 }
                  # 3xx/4xx response
                  if ($tx->error) {
                      my $err = $tx->error;
                      $c->app->log->error("Searchd error: $err->{message}");

                      my $code = $tx->res->code == 400 ? 400 : 500;
                      my $rs = $tx->res->json;
                      if (defined $rs && $rs->{error} && $rs->{error}->{message}) {
                          $message = $rs->{error}->{message};
                      }
                      else {
                          $message = $err->{message};
                      }

                      $c->render(query_words => $query,
                                 target => $target,
                                 sort_reverse => $sort_reverse,
                                 message => $message,
                                 error => 1,
                                 status => $code,
                                );
                      $c->rendered($code);
                      return;
                  }

                  my $result = $tx->result->json;
                  if (!defined $result) {
                      $c->app->log->error("searchd Error: receive invalid message");
                      $message = "searchd_error";
                      $c->render(query_words => $query,
                                 target => $target,
                                 sort_reverse => $sort_reverse,
                                 message => $message,
                                 error => 1,
                                 status => 500,
                                );
                      $c->rendered(500);
                      return;
                  }

                  # convert data
                  my $users = $c->model('users');
                  for my $item (@{$result->{hits}}) {
                      if ($item->{author}) {
                          if ($item->{author} == 1) {
                              $item->{author} = $users->anonymous_user;
                          } else {
                              $item->{author} = $users->select(uid => $item->{author});
                          }
                      }
                  }

                  $c->render(query_words => $query,
                             target => $target,
                             results => $result,
                             sort_reverse => $sort_reverse,
                             message => "",
                             error => 0,
                             start => $start,
                             item_per_page => $item_per_page,
                            );
                  $c->rendered();
              });
}


1;

