package Newslash::Web::Controller::Story;
use Mojo::Base 'Mojolicious::Controller';
use Mojo::Util qw(dumper);

sub single {
    my $c = shift;
    my $session = $c->session('session');
    my $sid = $c->stash('sid');
    my $stories = $c->model('stories');
    my $users = $c->model('users');
    my $user = $c->stash('user');

    my $story = $stories->select(sid => $sid,
                                 hide_future => !$user->{is_admin},
                                 public_only => !$user->{is_admin},
                                );

    #$c->app->log->debug(dumper($story));
    if (!$story) {
        $c->render('Not found', status => '404');
        return;
    }

    if ($story->{public} eq 'no' && !$user->{is_admin}) {
        $c->render('Not found', status => '404');
        return;
    }

    my $discussions = $c->model('discussions');
    my $discuss = $discussions->select(sid => $sid);
    my $d_id = $discuss->{id};
    my $comments = [];

    if ($d_id) {
        $comments = $c->model('comments')->select(discussion_id => $d_id);
        $c->model('comments')->build_comment_tree($comments);
    }

    my $related = $stories->get_related_items(stoid => $story->{stoid});
    my $histories = [];
    for my $i (@$related) {
        if ($i->{type} eq "submission") {
            push @$histories, $i;
        }
    }
    #$c->app->log->debug(dumper($related));

    my $params = $stories->parameters($story->{stoid});
    #$c->app->log->debug(dumper($params));

    # this query not fast, because uses multiple queries to get config and prefs
    # my $authors = $users->select(author => 1);
    my $authors = $c->ccache->model("users")->select(author => 1);

    my $mod_reasons = $c->model('moderations')->reasons;
    $c->stash(moderate_reasons => $mod_reasons);

    my $page = { type => "single",
                 content_type => "story",
                 stoid => $story->{stoid},
                 sid => $story->{sid},
                 id => $story->{stoid},
               };

    $c->render(item => $story,
               discussion => $discuss,
               comments => $comments,
               related => $related,
               histories => $histories,
               params => $params,
               authors => $authors,
               use_captcha => 1,
               page => $page,
              );

    $c->stats->increment_story_pageview_counter($story->{stoid});

}

1;

