package Newslash::Web::Controller::Tag;
use Mojo::Base 'Newslash::Web::Controller';
use List::Util qw(any);



sub list_tags {
    my $c = shift;
    my $topics = $c->model('topics')->select();

    my $page = { type => "tag_list",
                 content_type => "topics",
               };

    $c->render("tag/tag_list",
               items => $topics,
               page => $page,
              );
    return;
    $c->stats->add_event_counter("taglist_view");

}

sub list_tagged_items {
    my $c = shift;
    my $user = $c->stash('user');
    my $tag_name = $c->stash('tagname');
    my $type = $c->stash('type');

    # check parameter
    my $page_num = $c->stash('page') || 1;
    my $config = $c->app->config("Timeline") || {};
    my $limit = $config->{item_per_page} || 10;
    my $offset = $limit * ($page_num - 1);

    if ($page_num !~ m/^\d+$/
        || $page_num < 1
        || $type && !any { $_ eq $type } qw(story journal submission)) {
        $c->rendered(404);
        return;
    }


    # get tagname id
    my $tagname_ids = [];
    my $tags = $c->model('tags');
    my $topic_name = $tag_name;

    # first, check topics table
    my $topics = $c->model('topics')->select(OR => {textname => $tag_name, keyword => $tag_name});
    if ($topics && @$topics) {
        # topic's tid is same as tagname id
        push @$tagname_ids, $topics->[0]->{tid};

        # if tagname corresponds keyword, redirect
        if ($topics->[0]->{textname} eq $tag_name) {
            if ($type) {
                $c->redirect_to("/tag/$topics->[0]->{keyword}/$type", 301);
            }
            else {
                $c->redirect_to("/tag/$topics->[0]->{keyword}/", 301);
            }
            return;
        }
        $topic_name = $topics->[0]->{textname};
        $tag_name = $topics->[0]->{keyword};
    }
    my $rs = $tags->select(tag_name => $tag_name);

    if (!$rs) {
        $c->rendered(500);
        return;
    }
    if (!@$rs) {
        $c->render('Not found', status => '404');
        return;
    }
    push @$tagname_ids, $rs->[0]->{tagnameid};

    my $show_future = $user->{is_admin} ? 1 : 0;
    my $show_nonpublic = $user->{is_admin} ? 1 : 0;

    my $items = $c->model('firehose')->select(type => $type,
                                              tag_name_id => $tagname_ids,
                                              order_by => {create_time => "DESC"},
                                              limit => $limit + 1,
                                              offset => $offset,
                                              maintable => { ne => "preview"},
                                              hide_future => !$show_future,
                                              public_only => !$show_nonpublic,
                                             );
    if (!$items) {
        $c->rendered(500);
        return;
    }

    for my $item (@$items) {
        $item->{fhid} = $item->{id};
        $item->{firehose_id} = $item->{id};
        $item->{id} = $item->{srcid};
    }

    my $page = { type => "tag",
                 tag_name => $topic_name,
                 tag_keyword => $topics->[0]->{keyword},
                 content_type => $type,
                 number => $page_num,
                 has_prev => @$items > $limit,
                 target => $c->param('type') || "",
               };

    # query $limit + 1 items, so pop last item
    pop @$items;
    $c->render("tag/base",
               items => $items,
               page => $page,
              );
    return;

    $c->stats->add_event_counter("tagpage_view");
}


1;

