/* article-editor.js */

var articleEditor = {
  _initialized: 0,
  instances: [],
};

articleEditor.init = function init () {
  if (this._initialized) { return; }
  
  function data() {
    var data = {
      // only properties of editor will be send when preview/post 
      editor: {
        title: "",
        intro_text: "",
        body_text: "",
        create_time: "",
        author: "",
        dept: "",
        comment_status: "enabled",
        submissioncopy: 0,
        url: "",
        email: "",
        tags_string: "",
        related_urls: "",
        display: 1,
        posttype: 1,
        formatter: "modern",
        id: 0,
	source_id: 0,
	source_type: 0,
      },

      message: "",
      createdUrl: "",
      currentTopics: [],
      
      enableAutoPreview: 1,
      mode: "editing",

      primaryTopicIconURL: "",
      isUpdate: 0,
    };
    return data;
  }

  var props = {
    item: {
      type: Object,
      default: function () {return new Content();},
    },
    contentType: { type: String, required: true },
    contentId: { type: Number, dafault: 0 },
    sourceType: { type: String },
    sourceId: { type: Number, default: 0 },
    csrfToken: { type: String },
  };

  /*
   * watch
   */
  var watch = {
    'editor.intro_text': function watchIntrotext(val, oldVal) {
      this.item.intro_text = (val.length > 0) ? _quoteHtml(val) : "";
    },
    'editor.body_text': function watchBodytext(val, oldVal) {
      this.item.body_text = (val.length > 0) ? _quoteHtml(val) : "";
    },
    'editor.title': function watchTitle(val, oldVal) {
      this.item.title = (val.length > 0) ? _quoteTitle(val) : "";
    },
    'editor.url': function watchURL(val, oldVal) {
      this.item.url = (val.length > 0) ? val : "";
    },
    'editor.create_time': function watchCreatetime(val, oldVal) {
      this.item.time_string = newslash.util.formatToLocalDateTime(new Date(val));
    },
    'editor.author': function watchAuthor(val, oldVal) {
      this.item.author = val;
    },
    'editor.dept': function watchDept(val, oldVal) {
      this.item.dept = val;
    },
    'editor.tags_string': _updateTopics,
  };

  function _quoteHtml(html) {
    return newslash.util.quoteHtml(html, siteConfig.editorConfig.allowed_tags);
  }

  function _quoteTitle(html) {
    return newslash.util.quoteTitle(html, {});
  }

  /*
   * computed
   */
  var computed = {};


  /*
   * methods and relative private functions
   */
  var methods = {
    showPreview: _showPreview,
    postItem: _postItem,
    leavePreview: function leavePreview(event) {
      this.mode = "editing";
      this.item.intro_text = _quoteHtml(this.editor.intro_text);
      this.item.body_text = _quoteHtml(this.editor.body_text);
      this.item.title = _quoteTitle(this.editor.title);
    },
    startEdit: function startEdit() {
      _startEdit.call(this);
    },
    cancelEdit: function cancelEdit() {
      for (k in this.originalItem) {
        this.item[k] = this.originalItem[k];
      }
      this.mode = "show_all";
    },
    updateTopics: _updateTopics,
    topicIconURL: function topicIconURL(topic) {
      return newslash.util.topicIconURL(topic);
    },
    updatePrimaryTopicIconURL: function updatePrimaryTopicIconURL() {
      if (this.item.primary_topic) {
        this.primaryTopicIconURL = newslash.util.topicIconURL(this.item.primary_topic.keyword);
        return;
      }
      this.primaryTopicIconURL = "";
    },
  };


  function _showPreview(event) {
    var postData = _preparePostData.call(this, "preview");

    if (this.contentType == "journal") {
      newslash.previewJournal(postData).then(
        (resp) => { // success
          this.message = "";
          this.item.title = resp.item.title;
          this.item.intro_text = resp.item.intro_text;
          this.item.url = this.editor.url;
          this.item.primary_topic = resp.item.topic;
          this.updatePrimaryTopicIconURL();

          this.mode = "preview";
        },
        (resp) => { // fail
          if (resp.message) {
            this.message = resp.message;
          }
        }
      );
      return;
    }

    if (this.contentType == "story") {
      newslash.admin.previewStory(postData).then(
        (resp) => { // success
          this.message = "";
          this.item.title = resp.item.title;
          this.item.intro_text = resp.item.intro_text;
          this.item.url = this.editor.url;
          this.item.primary_topic = resp.item.topic;
          this.updatePrimaryTopicIconURL();

          this.mode = "preview";
        },
        (resp) => { // fail
          if (resp.message) {
            this.message = resp.message;
          }
        }
      );
      return;
    }

    if (this.contentType == "submission") {
      newslash.previewSubmission(postData).then(
        (resp) => { // success
          this.message = "";
          this.item.title = resp.item.title;
          this.item.intro_text = resp.item.intro_text;
          this.item.url = this.editor.url;
          this.item.primary_topic = resp.item.topic;
          this.updatePrimaryTopicIconURL();

          this.mode = "preview";
        },
        (resp) => { // fail
          if (resp.message) {
            this.message = resp.message;
          }
        }
      );
      return;
    }

    console.log("no contentType defined!");
  }

  function _postItem(event) {
    this.mode = "posting";
    var postData = _preparePostData.call(this, "post");

    // journal post
    if (this.contentType == "journal") {
      postData.csrf_token = this.csrfToken;
      newslash.postJournal(postData).then(
        (resp) => { // success
          this.message = "";
          var url = '/~' + resp.author + '/journal/' + rsep.id;
          
          // check if new post
          if (!postData.id) {
            this.createdUrl = url;
	    this.editor.id = id;
          }
          this.message = "";
          this.mode = "posted";
        },
        (resp) => { // fail
          this.message = resp.message;
          this.mode = "preview";
        }
      );
      return;
    }

    // submission post
    if (this.contentType == "submission") {
      postData.csrf_token = this.csrfToken;
      newslash.postSubmission(postData).then(
        (resp) => { // success
          this.message = "";
          var url = '/submission/' + resp.id;
          
          // check if new post
          if (!postData.id) {
            this.createdUrl = url;
	    this.editor.id = id;
          }
          this.message = "";
          this.mode = "posted";
        },
        (resp) => { // fail
          this.message = resp.message;
          this.mode = "preview";
        }
      );
      return;
    }

    // story post
    if (this.contentType == "story") {
      postData.csrf_token = this.csrfToken;
      newslash.admin.postStory(postData).then(
        (resp) => { // success
          this.message = "";
          var url = '/story/' + resp.sid;
          
          // check if new post
          if (!postData.id) {
            this.createdUrl = url;
	    this.editor.id = resp.id;
          }
          this.message = "";
          this.mode = "posted";
        },
        (resp) => { // fail
          this.message = resp.message;
          this.mode = "preview";
        }
      );
      return;
    }
    
  }

  function _preparePostData(action) {
    action = action || "preview";

    var postData = {};
    postData.item = {};
    postData.action = action;
    
    if (articleEditor.recaptchaToken) {
      postData.recaptcha_token = articleEditor.recaptchaToken;
    }

    Object.keys(this.editor).forEach(k => {
      postData.item[k] = this.editor[k];
    });

    // prepare sourceId
    if (this.contentType == "story") {
      if (this.sourceType == "submission") {
        postData.item.submission_id = "this.sourceId";
      } else if (this.sourceType == "journal") {
        postData.item.journal_id = "this.sourceId";
      }
    }

    return postData;
  }

  function _updateTopics() {
    var tags = this.editor.tags_string.split(" ");
    this.currentTopics.splice(0);
    if (tags.length == 0) {
      return;
    }
    var pushed = {};
    tags.forEach((tag) => {
      if (siteConfig.topics[tag]) {
        var keyword = siteConfig.topics[tag].keyword;
        if (!pushed[keyword]) {
          this.currentTopics.push(keyword);
          pushed[keyword] = 1;
        }
      }
    });
  }

  /*
   * created and related private functions
   */
  function created() {
    _startEdit.call(this);
  }

  function _startEdit() {
    this.mode = "editing";
    // backup original item
    // copy item property to c (component)
    this.originalItem = {};
    for (k in this.$data.item) {
      if (this.item[k]) {
        this.originalItem[k] = this.item[k];
      }
    }

    // load item data
    if (this.contentType == "journal") {
      if (this.contentId) {
        _loadJournal.call(this);
      } else {
        // create new journal
        this.mode = "editing";
      }
      return;
    }

    if (this.contentType == "story") {
      if (this.contentId) {
        _loadStory.call(this);
      } else {
        if (this.sourceType == "submission") {
          _loadStoryFromSubmission.call(this);
        } else if (this.sourceType == "journal") {
          _loadStoryFromJournal.call(this);
        } else if (this.sourceType == "story") {
          _loadStory.call(this);
        } else {
          // create new story
          _createNewStory.call(this);
        }
      }
      return;
    }
   
    if (this.contentType == "submission") {
      // create new submission
      this.mode = "editing";
      return;
    }

  }

  function _loadJournal() {
    newslash.getJournal(this.contentId).then(
      resp => { // succeed
        var item = resp.journal;
        this.isUpdate = 1;
        this.editor.title = item.title;
        this.editor.intro_text = item.article;
        this.editor.tags_string = item.tags_string || "";
        this.editor.comment_status = item.comment_status;
        this.editor.formatter = item.formatter;
        this.editor.posttype = item.posttype;
        this.editor.id = item.id;
          
        // check formatter
        if (this.editor.formatter == "legacy") {
          this.enableAutoPreview = 0;
        }
          
        // start editing
        this.updatePrimaryTopicIconURL();
        this.mode = "editing";
      },
      resp => { // failed
        this.message = resp.body.message;
      }
    );
  }

  function _createNewStory() {
    // set createtime
    var dt = new Date((new Date()).getTime() + (30 * 60 * 1000));
    this.editor.createtime = newslash.util.formatToLocalISOString(dt);
    this.editor.author = user.nickname;

    // start editing
    this.mode = "editing";
    return;
  }

  function _prepareStory(item) {
    item.content_type = 'story';
    for (var k in this.editor) {
      if (item[k] !== undefined) {
        this.editor[k] = item[k];
      }
    }
    // convert tags to tags_string
    if (item.tags) {
      var tagnames = item.tags
	  .filter(tag => (tag.private == "no"
			  && tag.tagname != "submission"
			  && tag.tagname != "journal"
			 ))
	  .map(tag => tag.tagname);
      this.editor.tags_string = tagnames.join(" ");
      this.updateTopics();
    }

    // TODO: related stories
    // convert public to display
    this.editor.display = (item.public == "no") ? 0 : 1;
    if (item.create_time) {
      this.editor.createtime = newslash.util.formatToLocalISOString(newslash.util.decodeMySQLDateTime(item.create_time));
    } else {
      var dt = new Date((new Date()).getTime() + (30 * 60 * 1000));
      this.editor.createtime = newslash.util.formatToLocalISOString(dt);
    }
    this.editor.id = item.stoid;
    this.editor.author = user.nickname;
    this.editor.source_id = this.sourceId || 0;
    this.editor.source_type = this.sourceType || "";
  }

  function _loadStory() {
    newslash.admin.getStory(this.sourceId).then(
      resp => {
        _prepareStory.call(this, resp.item);
      },
      fail => {
        this.message = fail.body.message;
        console.log("story fetch failed");
      }
    );
  }

  function _loadStoryFromSubmission() {
    newslash.admin.getStoryFromSubmission(this.sourceId).then(
      resp => {
        _prepareStory.call(this, resp.item);
      },
      fail => {
        this.message = fail.body.message;
        console.log("story fetch failed");
      }
    );
  }

  function _loadStoryFromJournal() {
    newslash.admin.getStoryFromJournal(this.sourceId).then(
      resp => {
        _prepareStory.call(this, resp.item);
      },
      fail => {
        this.message = fail.body.message;
        console.log("story fetch failed");
      }
    );
  }


  /*
   * register component
   */
  this._component = Vue.component('article-editor', {
    template: '#article-editor-template',
    props: props,
    data: data,
    computed: computed,
    watch: watch,
    methods: methods,
    created: created,
  });

  this._initialized = 1;
};

/*
 * utilities
 */
  
articleEditor.ArticleEditor = class ArticleEditor {
  constructor(targetElement) {
    if (!targetElement) {
      console.error('error in ArticleEditor(): no element given');
      return;
    }
    this.vm = new Vue({el: targetElement});
  }
};

articleEditor.addTrigger = function addTrigger (selector, targetElement) {
  if (!targetElement) {
    console.error('error in articleEditor.addTrigger(): no element given');
    return;
  }
  if (!selector) {
    console.error('error in articleEditor.addTrigger(): no selector given');
    return;
  }

  var els = document.querySelectorAll(selector);
  for (var i = 0; i < els.length; i++) {
    els[i].addEventListener('click', function (e) {
      articleEditor.init();
      articleEditor.instances.push(new articleEditor.ArticleEditor(targetElement));
    });
  }
};


