/* boxes-editor.js */
const boxesEditor = {};

boxesEditor.run = function run (params) {

  var vm;

  Vue.component('box-editor', {
    template: '#box-editor',
    props: {
      box: Object,
    },
    data: function () {
      return { message: "", };
    },
    computed: {
      nameNg: function () { return !this.box.name.length; },
    },
    methods: {
      save: function save() {
        // create/update box
        this.message = "";
        newslash.admin.updateBox(this.box).then(
          (response) => { // success
            if (response.error) {
              this.message = "error: " + response.message;
              return;
            }
            if (!response.id) {
              this.message = "error: " + response.message;
              return;
            }
            this.message = "saved.";
            if (this.box.id == 0) {
              this.box.id = response.id;
            }
            this.box.edited = false;
          },
          (response) => { // fail
            this.message = "error";
          }
        );
      },
      close: function close() {
        this.box.editing = false;
        this.message = "";
        if (this.box.id == 0) {
          vm.boxItems.pop();
        }
      },
    },
  });

  const data = {
    message: '',
    boxItems: [],
  };
  const computed = {};
  const methods = {
    editItem: function editItem(item) {
      item.editing = true;
      item.edited = true;
    },
    deleteItem: function deleteItem() {
      const target = [];
      for (var i = 0; i < this.boxItems.length; i++) {
        if (this.boxItems[i].selected) {
          target.push({id: this.boxItems[i].id, index: i});
        }
      }          
      if (target.length == 0) {
        return;
      }
      const msg = "delete " + target.length + " item(s)?";
      const result = window.confirm(msg);
      if (result) {
        // delete items
        target.reverse();
        target.forEach(item => {
          newslash.admin.deleteBox(item.id).then(
            (response) => { // success
              if (response.error) {
                this.message = "error: " + response.message;
                return;
              }
              this.boxItems.splice(item.index, 1);
            },
            (response) => { // fail
              this.message = "error";
            }
          );
        });
      }
    },
    newItem: function newItem() {
      const item = {
        id: 0,
        name: 'new item',
        comment: '',
        model: '',
        query_params: '',
        limit: '',
        template: '',
        selected: false,
        editing: true,
        edited: true,
      };
      const it = this.boxItems.push(item);
    },
  };

  function initItems (items) {
    items.forEach(item => {
      Vue.set(item, 'selected', false);
      Vue.set(item, 'editing', false);
      Vue.set(item, 'edited', false);
    });
  };

  vm = new Vue({el: params.el,
                data: data,
                computed: computed,
                methods: methods,
                created: function () {
                  newslash.admin.getBoxes().then(
                    (resp) => { // success
                      this.boxItems = resp.items || [];
                      this.message = resp.message || '';
                      initItems(this.boxItems);
                    },
                    (resp) => { // fail
                      this.boxItems = [];
                      this.message = resp.message || "failed to get items";
                    }
                  );
                },
               });
};
