/* feeds-editor.js */
var feedsEditor = {};

feedsEditor.run = function run (params) {
  var vm;

  Vue.component('feed-editor', {
    template: '#feed-editor',
    props: {
      feed: Object,
    },
    data: function () {
      return { message: "" };
    },
    methods: {
      save: function save() {
        // create/update feed
        this.message = "";
        var postData = {
          action: this.feed.feed_id ? "update" : "create",
          url: this.feed.url,
          title: this.feed.title,
          name: this.feed.name,
          description: this.feed.description,
          feed_id: this.feed.feed_id,
        };
        this.$http.post("/api/v1/admin/feed", postData).then(
          (response) => { // success
            if (response.body.error) {
              this.message = "error: " + response.body.message;
              return;
            }
            if (!response.body.feed_id) {
              this.message = "error: " + response.body.message;
              return;
            }
            this.message = "saved.";
            if (this.feed.feed_id == 0) {
              this.feed.feed_id = response.body.feed_id;
            }
          },
          (response) => { // fail
            this.message = "error";
            return;
          }
        );
      },
      close: function close() {
        this.feed.editing = false;
        this.message = "";
        if (this.feed.feed_id == 0) {
          vm.feeds.pop();
        }
      },
      fetch: function fetch() {
        this.message = "";
        var postData = {
          action: "fetch",
          url: this.feed.url,
        };
        this.$http.post("/api/v1/admin/feed", postData).then(
          (response) => { // success
            if (response.body.error) {
              this.message = "error: " + response.body.message;
              return;
            }
            this.feed.url = response.body.item.url;
            this.feed.title = response.body.item.title;
            this.feed.description = response.body.item.description;
          },
          (response) => { // fail
            this.message = "error: " + response.body.message;
            return;
          }
        );
        
      },
    },
  });

  var data = {
    message: '',
    feeds: [],
  };
  var computed = {};
  var methods = {
    editItem: function editItem(item) {
      Vue.set(item, 'editing', true);
    },
    deleteItem: function deleteItem() {
      var target = [];
      for (var i = 0; i < this.feeds.length; i++) {
        if (this.feeds[i].selected) {
          target.push({id: this.feeds[i].feed_id, index: i});
        }
      }          
      if (target.length == 0) {
        return;
      }
      var msg = "delete " + target.length + " item(s)?";
      var result = window.confirm(msg);
      if (result) {
        // delete items
        target.reverse();
        target.forEach(item => {
          this.$http.post("/api/v1/admin/feed", {id: item.feed_id}).then(
            (response) => { // success
              if (response.body.error) {
                this.message = "error: " + response.body.message;
                return;
              }
              this.feeds.splice(item.index, 1);
            },
            (response) => { // fail
            }
          );
        });
      }
    },
    newItem: function newItem() {
      var item = {
        id: 0,
        title: '',
        description: '',
        url: '',
        selected: false,
        editing: true,
      };
      var it = this.feeds.push(item);
    },
  };

  function initItems (items) {
    items.forEach(item => {
      Vue.set(item, 'selected', false);
      Vue.set(item, 'editing', false);
    });
  };

  vm = new Vue({el: params.el,
                data: data,
                computed: computed,
                methods: methods,
                created: function () {
                  this.$http.get("/api/v1/admin/feed").then(
                    (resp) => { // success
                      this.feeds = resp.body.items || [];
                      this.message = resp.body.message || '';
                      initItems(this.feeds);
                    },
                    (resp) => { // fail
                      this.feeds = [];
                      this.message = resp.body.message || "failed to get items";
                    }
                  );
                },
               });
};
