/* filters-editor.js */
var filtersEditor = {};

filtersEditor.run = function run (params) {

  var vm;
  var data = {
    message: '',
    state: '',
    filters: [],
  };

  var computed = {};
  var methods = {
    deleteItem: function deleteItem(filter) {
      newslash.admin.deleteFilter(filter.values.filter_id).then(
        (response) => { // success
          if (response.error) {
            this.message = response.message;
            this.state = "error";
            return;
          }
          this.state = "deleted";
          const index = this.filters.findIndex(filter);
          if (index > -1) {
            this.filters.splice(index, 1);
          } else {
            this.state = "error";
          }
        },
        (response) => { // fail
          this.state = "error";
          if (response && response.error) {
            this.message = response.message;
          }
        }
      );
    },
    saveItem: function saveItem(filter) {
      newslash.admin.updateFilter(filter.values).then(
        (response) => { // success
          if (response.error || !response.filter_id) {
            this.message = response.message;
            this.state = "error";
            return;
          }
          this.state = "saved";
          filter.editing = 0;
          if (!filter.values.filter_id) {
            filter.values.filter_id = response.filter_id;
          }
        },
        (response) => { // fail
          this.state = "error";
          if (response && response.error) {
            this.message = response.message;
          }
        }
      );
    },
    editItem: function editItem(filter) {
      filter.editing = 1;
    },
    cancelEdit: function cancelEdit(filter) {
      filter.editing = 0;
    },
    newItem: function newItem() {
      const newItem = { editing : 1,
                        checked: 0,
                        values: { filter_id: 0,
                                  name: "",
                                  description: "",
                                  message: "",
                                  status: "enabled",
                                  regexp: "",
                                  ratio: 0,
                                  threshold: 0,
                                  limit: 0,
                                  pathes: "",
                                  fields: "" },
                      };
      this.filters.push(newItem);
    },
  };

  vm = new Vue({el: params.el,
                data: data,
                computed: computed,
                methods: methods,
                created: function () {
                  newslash.admin.getFilters().then(
                    (resp) => { // success
                      if (resp.items) {
                        resp.items.forEach(item => {
                          this.filters.push({ editing: 0,
                                              checked: 0,
                                              values: item });
                        });
                        this.message = resp.message || '';
                      }
                    },
                    (resp) => { // fail
                      this.filters = [];
                      this.message = resp.message || "failed to get items";
                    }
                  );
                },
               });
};
