/* timeline.js */
var timeline = {};

timeline.run = function (params) {
  /* define exotic parameters */
  params = params || {};
  var userConfig = params.userConfig || {};
  var siteConfig = params.siteConfig || {};
  var pageInfo = params.pageInfo || {};
  var user = params.user || {};

  if (!params.el) {
    console.log('error in commentTree.run(): no element given');
    return;
  }

  /*
   * register <timeline-pager>
   */
  Vue.component('timeline-pager', {
    template: '#timeline-pager-template',
    props: { previousInfo: Object },
    data: function () { return {}; },
    computed: {
      prevUrl: function () {
        var prevInfo = this.previousInfo;
        if (prevInfo
            && prevInfo.year
            && prevInfo.month
            && prevInfo.day
            && prevInfo.id
            && prevInfo.content_type) {
          return "/" + prevInfo.content_type
            + newslash.pathFromDate(prevInfo.year, prevInfo.month, prevInfo.day)
            + "#" + prevInfo.id;
        }
        return "";
      },
    },
    created: function () { return; },
  });

  /*
   * register <timeline-vote>
   */
  Vue.component('timeline-vote', {
    template: '#timeline-vote-template',
    props: { item: Object },
    data: function () { return {}; },
    created: function () { return; },
    methods: {
      vote: function vote(voteType) {
        var globjid = this.item.globjid;
        if (voteType == this.item.tagmoderation) {
          // cancel vote
          newslash.cancelVote(globjid, voteType).then(
            (resp) => { // success
              this.item.tagmoderation = "";
              console.log("vote " + voteType + " for " + globjid + " succeeded. tag_id: " + resp.result.tag_id );
            },
            (resp) => { // fail
              console.log("vote " + voteType + " for " + globjid + " failed.");
            }
          );
          return;
        }
        newslash.vote(globjid, voteType).then(
          (resp) => { // success
            this.item.tagmoderation = voteType;
            console.log("vote " + voteType + " for " + globjid + " succeeded. tag_id: " + resp.result.tag_id );
          },
          (resp) => { // fail
            console.log("vote " + voteType + " for " + globjid + " failed.");
          }
        );
      },
    }
  });

  /*
   * register <timeline-item>
   */
  Vue.component('timeline-item', {
    template: '#timeline-item-template',
    props: {item: Object },
    data: function () { return { user: vm.user }; },
    created: function () { return; },
  });

  /*
   * register <timeline-filter-ui>
   */
  Vue.component('timeline-filter-ui', {
    template: '#timeline-filter-ui-template',
    props: {
      loading: Boolean,
      message: String,
      initDateRange: String,
      initThreshold: Number,
      target: String,
    },
    data: function () {
      return {
        threshold: 1,
        dateRange: "",
      };
    },
    created: function () {
      this.dateRange = this.initDateRange || "";
      this.threshold = this.initThreshold === undefined ? "1" : this.initThreshold;
    },
    methods: {
      setThreshold: function setThreshold (threshold) {
        if (this.threshold != threshold) {
          this.threshold = threshold;
          vm.$emit('updateTimeline', {threshold: threshold, dateRange: this.dateRange});
        }
      },
      setDateRange: function setDateRange () {
        vm.$emit('updateTimeline', {threshold: this.threshold, dateRange: this.dateRange});
      },
    },
  });

  function updateTimeline(vm, options) {
    vm.loading = true;
    vm.message = "";
    newslash.getTimeline(vm.target, options).then(
      (resp) => { // success
        vm.items = resp.result.items;
        vm.previousInfo = resp.result.prev_info || {};
        vm.loading = false;
      },
      (resp) => { // fail
        vm.loading = false;
        vm.items = [];
        vm.previousInfo = {};
        if (resp.error && resp.error.message) {
          vm.message = resp.error.message;
        } else {
          vm.message = "update_timeline_error";
        }
      }
    );
  }

  var vm = this.vm = new Vue({
    el: params.el,
    data: {
      items: [],
      loading: false,
      message: "",
      previousInfo: {},
      user: params.user,
      preLoading: true,
      initDateRange: params.dateRangeKey,
      initThreshold: (params.threshold !== undefined) ? params.threshold : 1,
      target: params.target,
    },
    created: function created() {
      updateTimeline(this, {threshold: this.initThreshold, dateRange: this.initDateRange});
      this.preLoading = false;
    },
  });

  vm.$on("updateTimeline", function (args) {
    updateTimeline(this, args);
  });
};

