/* newslash support plugin for Vue.js */

function VueNewslash(vm) {
  this.vm = vm;
};

VueNewslash.prototype.postUrlOf = {
  submission: '/api/v1/submission',
  story: '/api/v1/story',
  journal: '/api/v1/journal',
  comment: '/api/v1/comment',
  user: '/api/v1/user',
  moderate: "/api/v1/moderation",
  metamoderate: "/api/v1/metamoderation",
  poll: "/api/v1/poll",
  vote: "/api/v1/vote",
};

VueNewslash.prototype.executeCaptcha = function executeCaptcha(onSuccess, onFail) {
  if (typeof grecaptcha === 'undefined' || !grecaptcha) {
    console.warn("$newslash.executeCaptcha: no grecaptcha!");
    onFail({message: "no_grecaptcha"});
    return;
  }

  if (statusBar) {
    statusBar.init();
    statusBar.loading("checking if you are a bot...");
  }
  VueNewslash.currentRecaptchaCallback = function recaptchaCallback(token) {
    if (token) {
      if (statusBar) { statusBar.hide(); }
      onSuccess(token);
    } else {
      if (statusBar) { statusBar.error("captcha_error"); }
      onFail({resp: "captcha_error"});
    }
  };
  grecaptcha.reset();
  grecaptcha.execute();
};


VueNewslash.prototype.post = function post(type, data, options, onSuccess, onFail) {
  var url = this.postUrlOf[type];
  if (!url) {
    console.error("$newslash.post: invalid type");
    onFail({message:"invalid_type"});
    return;
  }

  options = options || {};
  if (typeof options === "function") {
    onFail = onSuccess;
    onSuccess = options;
    options = {};
  }

  var csrfToken = options.csrfToken || "";
  var noCaptcha = options.noCaptcha || 0;

  // check captcha
  if (typeof grecaptcha !== 'undefined'
      && !noCaptcha
      && (!user || !user.is_login)) {
    this.executeCaptcha(
      (token) => {
        data.recaptcha_token = token;
        _doPost(this.vm, url, data, csrfToken, 0, onSuccess, onFail);
      },
      (resp) => {
        console.error("$newslash.post: captcha check failed.");
        onFail(resp);
      });
    return;
  }
  _doPost(this.vm, url, data, csrfToken, 0, onSuccess, onFail);

  function _doPost(vm, url, data, newToken, retry, onSuccess, onFail, lastResp) {
    if (retry > 2) {
      onFail(lastResp);
      return;
    }

    if (newToken) { data.csrf_token = newToken; }

    vm.$http.post(url, data).then(
      (resp) => { //success
        onSuccess(resp);
        return;
      },
      (resp) => { //fail
        if (resp.body && resp.body.reason == "invalid_csrf_token") {
          var newToken = resp.body.csrf_token;
          if (newToken) {
            // retry with new token
            _doPost(vm, url, data, newToken, retry + 1, onSuccess, onFail, resp);
            return;
          }
        }
        onFail(resp);
        return;
      });
  }
};

VueNewslash.prototype.getUrlOf = {
  submission: '/api/v1/submission',
  story: '/api/v1/story',
  journal: '/api/v1/journal',
  comment: '/api/v1/comment',
  moderation: "/api/v1/moderation",
  metamoderation: "/api/v1/metamoderation",
  user: "/api/v1/user",
};

/* deprecated!
VueNewslash.prototype.getSubmissionByID = function getSubmissionByID(subid, onSuccess, onFail) {
  var url = this.getUrlOf.submission + "?subid=" + subid;
  this.vm.$http.get(url).then(onSuccess, onFail);
};
*/

/* deprecated!
VueNewslash.prototype.getStoryByID = function getStoryByID(stoid, onSuccess, onFail) {
  var url = this.getUrlOf.story + "?stoid=" + stoid;
  this.vm.$http.get(url).then(onSuccess, onFail);
};
*/

VueNewslash.prototype.getComments = function getComments(discusID, onSuccess, onFail) {
  if (typeof discusID === "function") {
    onFail = onSuccess;
    onSuccess = discusID;
    discusID = page.discussion_id || 0;
  }
  if (!discusID) {
    onFail({});
    return;
  }
  var url = this.getUrlOf.comment + "?discussion_id=" + discusID;
  this.vm.$http.get(url).then(onSuccess, onFail);
};

VueNewslash.prototype.getModerations = function getModerations(cid, onSuccess, onFail) {
  var url = this.getUrlOf.moderation + "?cid=" + cid;
  this.vm.$http.get(url).then(onSuccess, onFail);
};

/*
 * class functions
 */
VueNewslash.install = function install (Vue, options) {
  Object.defineProperties(Vue.prototype, {
    $newslash: {
      get: function getter() {
        return new VueNewslash(this);
      },
    },
  });
};

VueNewslash.init = function init () {
  if (typeof window !== 'undefined' && window.Vue) {
    window.Vue.use(this);
  }
};

VueNewslash.init();


/*
 * global function for hook
 */
function recaptchaDone(token) {
  if (!VueNewslash.currentRecaptchaCallback) {
    console.warn("recapcha: no VueNewslash.currentRecaptchaCallback!");
    return;
  }
  VueNewslash.currentRecaptchaCallback(token);
}
