# -*-Perl-*-
# AD codes-related  tests
use Mojo::Base -strict;

use Test::More;
use Test::Mojo;
use Data::Dumper;
use Newslash::Util::TestMan;

use POSIX qw(strftime);

my $t = Test::Mojo->new('Newslash::Web');
my $test_man = Newslash::Util::TestMan->new($t);
my ($admin, $user);
my $test_passwd = "foobar";

if ($t->app->mode eq 'test') {
    $admin = $test_man->create_admin("adtestadmin", $test_passwd);
    ok($admin, "create admin user");
}

subtest 'create/select/update/delete/exports AD codes' => sub {
    plan skip_all => "mode is not 'test'" if ($t->app->mode ne 'test');

    # login
    $t->post_ok('/my/login' => {Accept => '*/*'} => form => {nickname => $admin->{nickname}, passwd => $test_passwd})
      ->status_is(302);

    my $ad_codes = $t->app->model('ad_codes');

    # delete duplicated entry before
    for my $name (qw(test_ad_01 test_ad_02)) {
        my $ad_code = $ad_codes->select(name => $name);
        if ($ad_code) {
            my $rs = $ad_codes->delete(id => $ad_code->{ad_id});
        }
    }

    # create
    my $test_data = {
                     action => "create",
                     name => "test_ad_01",
                     description => "ADテスト1",
                     content => "＜広告コード01本体＞",
                    };

    $t->post_ok('/api/v1/admin/ad/code' => {Accept => '*/*'} => json => $test_data)
      ->status_is(200)
      ->json_has('/id');

    my $id = $t->tx->res->json->{id};
    if (!$id) {
        diag(Dumper($t->tx->res->json));
        return;
    }

    # select one
    $t->get_ok("/api/v1/admin/ad/code?ad_id=$id")
      ->status_is(200)
      ->json_has('/item')
      ->json_is('/item/ad_id', $id)
      ->json_is('/item/name', $test_data->{name})
      ->json_is('/item/description', $test_data->{description})
      ->json_is('/item/status', 'enabled')
      ->json_is('/item/content', $test_data->{content})
      ->json_has('/item/last_update');

    # select all
    $t->get_ok("/api/v1/admin/ad/code")
      ->status_is(200)
      ->json_has('/items')
      ->json_has('/items/0/ad_id')
      ->json_has('/items/0/name')
      ->json_has('/items/0/description')
      ->json_has('/items/0/status')
      ->json_has('/items/0/content')
      ->json_has('/items/0/last_update');

    # update
    my $update_req = { action => "update",
                       ad_id => $id,
                       content => "＜updated code＞",
                     };
    $t->post_ok('/api/v1/admin/ad/code' => {Accept => '*/*'} => json => $update_req)
      ->status_is(200)
      ->json_has('/id');

    # check updated item
    $t->get_ok("/api/v1/admin/ad/code?ad_id=$id")
      ->status_is(200)
      ->json_has('/item')
      ->json_is('/item/ad_id', $id)
      ->json_is('/item/name', $test_data->{name})
      ->json_is('/item/description', $test_data->{description})
      ->json_is('/item/status', 'enabled')
      ->json_is('/item/content', $update_req->{content})
      ->json_has('/item/last_update');

    # delete
    my $del_req = {
                   action => "delete",
                   id => $id
                  };
    $t->post_ok('/api/v1/admin/ad/code' => {Accept => '*/*'} => json => $del_req)
      ->status_is(200);

    # finish
    $t->get_ok('/my/logout')->status_is(302);
};

$test_man->cleanup;

done_testing();
