# -*-Perl-*-
use Mojo::Base -strict;
use Mojo::Date;

use Test::More;
use Test::Mojo;
use Mojo::Util qw(dumper);

use Newslash::Util::Test qw(create_user delete_user);

use POSIX qw(strftime);

my $t = Test::Mojo->new('Newslash::Web');

my $user;

# prepare for test
if ($t->app->mode eq 'test') {
    # at first, create test users.
    $user = create_user($t->app, "test01", "foobar");
    ok($user, "create test user");

}

subtest 'check password' => sub {
    # valid request
    $t->post_ok('/api/v1/login' => {Accept => '*/*'} => json => {nickname => 'test01', password => 'foobar'})
      ->status_is(200)
      ->content_type_like(qr/application\/json/)
      ->json_is('/nickname', 'test01')
      ->or(sub {diag "message: " . dumper($t->tx->res->json);});

    # invalid request 1
    $t->post_ok('/api/v1/login' => {Accept => '*/*'} => json => {nickname => 'test01', password => 'hogehoge'})
      ->status_is(401)
      ->json_is('/error', 1)
      ->json_has('/message')
      ->or(sub {diag "message: " . dumper($t->tx->res->json);});

    # invalid request 2
    $t->post_ok('/api/v1/login' => {Accept => '*/*'} => json => {nickname => 'test01'})
      ->status_is(401)
      ->json_is('/error', 1)
      ->json_has('/message')
      ->or(sub {diag "message: " . dumper($t->tx->res->json);});
};

#clean up
if ($user) {
    delete_user($t->app, "test01");
}

done_testing();
