# -*-Perl-*-
# destructive database-related  tests
use Mojo::Base -strict;
use Mojo::Date;

use Test::More;
use Test::Mojo;
use Data::Dumper;
use Newslash::Util::TestMan;

use POSIX qw(strftime);

my $t = Test::Mojo->new('Newslash::Web');
my $test_man = Newslash::Util::TestMan->new($t);
my $admin;
my $test_story;

if ($t->app->mode eq 'test') {
    $admin = $test_man->create_admin("story_test", "foobar");

    $test_story = $test_man->create_story( user => $admin,
                                           uid => $admin->{uid},
                                           title => "test story for firehose",
                                           dept => "test for firehose",
                                           introtext => '<p>foo bar hoge</p>',
                                           bodytext => '',
                                           topics_chosen => {49 => 10, 2271 => 20},
                                           commentstatus => '',
                                           tid => 49,
                                           time => "2020-11-11 10:00:00");
    ok($test_story, "create story for test");

}

SKIP: {
    skip "mode is not 'test'", 4 if ($t->app->mode ne 'test');
    $test_man->login($admin);

    subtest 'story post/update' => sub {

        my $test_title = "テストタイトル";
        my $test_introtext = <<EOT;
<p>insiderman曰く、</p><blockquote>

<p>はやくも来年発表されるとみられているiPhoneの新モデルの噂が出始めている（<a href="https://www.theverge.com/circuitbreaker/2017/11/13/16644266/apple-iphone-x-plus-rumor-two-fullscreen-phones-2018">The Verge</a>、<a href="http://jbpress.ismedia.jp/articles/-/51619">JBPress</a>、<a href="https://hardware.slashdot.org/story/17/11/14/000224/apple-could-launch-two-new-full-screen-iphones-next-year">Slashdot</a>）。</p>

<p>KGI SecuritiesのアナリストMing-Chi Kuo氏による予想では、6.5インチ有機ELディスプレイを搭載したモデルがiPhone Xの新たなモデル（「Plus」バージョン）として出るのではないか、とされている。また、iPhone 8 Plusと同等のサイズでiPhone Xのようにフルスクリーン化した6.1インチ液晶ディスプレイ搭載モデルも出る可能性があるとも予測されている。これは、「ハイエンド市場」のさまざまな需要を狙ったものになるという。iPhone Xはその価格も話題となったが、液晶ディスプレイの採用で価格を下げることができるようだ。</p>
</blockquote>
EOT
        my $related = "https://apple.srad.jp/story/" . $test_story->{sid} . "/";
        my $tidyed_introtext = $t->app->format_htmltext($test_introtext, "story");
        my $test_dept = "テストdept";
        my $createtime = strftime('%FT%T', gmtime);
        my $test_tags = "apple ios iphone hoge";
        my $test_data = {
                         action => 'preview',
                         item => {
                                  title => $test_title,
                                  createtime => $createtime,
                                  author => $admin->{nickname},
                                  dept => $test_dept,
                                  introtext => $test_introtext,
                                  bodytext => "",
                                  add_related => $related,
                                  tags_string => $test_tags,
                                  mediaurl => "",
                                  mediatype => "",
                                  commentstatus => "enabled",
                                  display => 1,
                                 },
                        };

        # preview
        $t->post_ok('/api/v1/admin/story' => {Accept => '*/*'} => json => $test_data)
          ->status_is(200)
          ->json_has('/item')
          ->json_is('/item/title' => $test_title);
        diag $t->tx->res->json->{message} if $t->tx->res->json && $t->tx->res->code != 200;

        # create
        $test_data->{action} = 'post';
        $t->post_ok('/api/v1/admin/story' => {Accept => '*/*'} => json => $test_data)
          ->status_is(200)
          ->json_has('/id')
          ->json_is('/type' => "story");
        diag $t->tx->res->json->{message} if $t->tx->res->code != 200;
        my $stoid = $t->tx->res->json->{id};
        ok($stoid, "get stoid for created story");
        return if !$stoid;

        # get
        $t->get_ok("/api/v1/admin/story?stoid=$stoid")
          ->status_is(200)
          ->json_has('/item')
          ->json_is('/item/stoid', $stoid)
          ->json_is('/item/title', $test_title)
          ->json_is('/item/intro_text', $tidyed_introtext)
          ->json_has('/item/related')
          ->json_is('/item/related/0/stoid', $test_story->{stoid})
          ->json_has('/item/tags/0')
          ->json_has('/item/tags/1')
          ->json_has('/item/tags/2')
          ->json_has('/item/tags/3')
          ->json_like('/item/tags/0/tagname', qr/(hoge|iOS|iPhone|アップル|apple)/)
          ->json_like('/item/tags/1/tagname', qr/(hoge|iOS|iPhone|アップル|apple)/)
          ->json_like('/item/tags/2/tagname', qr/(hoge|iOS|iPhone|アップル|apple)/)
          ->json_like('/item/tags/3/tagname', qr/(hoge|iOS|iPhone|アップル|apple)/)
          ->json_is('/item/dept', $test_dept);
        diag "message: " . $t->tx->res->json("/message") if $t->tx->res->code != 200;
        #diag "result: " . Dumper $t->tx->res->json;

        # update
        my $new_title = "テストタイトル2テストテスト";
        $test_data->{item}->{title} = $new_title;
        $test_data->{item}->{stoid} = $stoid;

        $t->post_ok('/api/v1/admin/story' => {Accept => '*/*'} => json => $test_data)
          ->status_is(200)
          ->json_has('/id')
          ->json_is('/type' => "story");
        #diag $t->tx->res->json->{message} if $t->tx->res->code != 200;

        # get updated item
        $t->get_ok("/api/v1/admin/story?stoid=$stoid")
          ->status_is(200)
          ->json_has('/item')
          ->json_is('/item/stoid', $stoid)
          ->json_is('/item/title', $new_title);

    };

    $test_man->logout;
}

$test_man->cleanup;
done_testing();
