# -*-Perl-*-
# timeline api tests
use Mojo::Base -strict;
use Mojo::Date;

use Test::More;
use Test::Mojo;
use Mojo::Util qw(dumper);

use Newslash::Util::TestMan;
    
my $t = Test::Mojo->new('Newslash::Web');
my $test_man = Newslash::Util::TestMan->new($t);

subtest 'get timeline' => sub {
    # create test data
    $test_man->create_submission(title => "submission-test",
				 intro_text => "test submission body");

    # get all items
    $t->get_ok("/api/v1/timeline?target=all")
      ->status_is(200)
      ->content_type_like(qr/application\/json/)
      ->json_has('/result')
      ->or(sub {diag "message: " . dumper($t->tx->res->json);});

    # get story items
    $t->get_ok("/api/v1/timeline?target=story")
      ->status_is(200)
      ->content_type_like(qr/application\/json/)
      ->json_has('/result')
      ->json_has('/result/items/0/stoid')
      ->or(sub {diag "message: " . dumper($t->tx->res->json);});

    # get comment items
    $t->get_ok("/api/v1/timeline?target=comment")
      ->status_is(200)
      ->content_type_like(qr/application\/json/)
      ->json_has('/result')
      ->json_has('/result/items/0/cid')
      ->or(sub {diag "message: " . dumper($t->tx->res->json);});

    # get journal items
    $t->get_ok("/api/v1/timeline?target=journal")
      ->status_is(200)
      ->content_type_like(qr/application\/json/)
      ->json_has('/result')
      ->json_has('/result/items/0/journal_id')
      ->or(sub {diag "message: " . dumper($t->tx->res->json);});

    # get submission items
    $t->get_ok("/api/v1/timeline?target=submission")
      ->status_is(200)
      ->content_type_like(qr/application\/json/)
      ->json_has('/result')
      ->json_has('/result/items/0/submission_id')
      ->or(sub {diag "message: " . dumper($t->tx->res->json);});

    # get poll items
    #$t->get_ok("/api/v1/timeline?target=poll")
    #  ->status_is(200)
    #  ->content_type_like(qr/application\/json/)
    #  ->json_has('/result')
    #  ->json_has('/result/0/qid')
    #  ->or(sub {diag "message: " . dumper($t->tx->res->json);});
};

$test_man->cleanup;
done_testing();
