# -*-Perl-*-
# basic simple and non-destructive tests
use Mojo::Base -strict;
use Mojo::Date;
use Mojo::Util qw(dumper);

use Test::More;
use Test::Mojo;
use Newslash::Util::Test qw(create_admin_user create_user delete_user);

my $t = Test::Mojo->new('Newslash::Web');

$t->get_ok('/')->status_is(200);
$t->get_ok('/journals')->status_is(200);
$t->get_ok('/my/login')->status_is(200);

$t->get_ok('/my/newuser')->status_is(200);
$t->get_ok('/journal/0/')->status_is(404);


SKIP: {
    skip "mode is 'test'", 5 if ($t->app->mode eq 'test');

    $t->get_ok('/story/16/01/28/0437240/')->status_is(200);
    $t->get_ok('/story/16/01/28/0')->status_is(404);

    $t->get_ok('/journal/600014/')->status_is(200);

    $t->get_ok('/submission/64070/')->status_is(200);
    $t->get_ok('/submission/64054/')->status_is(200);
}


TODO: {
    local $TODO = "not implemented";
    $t->get_ok('/story/16/01/28/0437240')->status_is(301);
    $t->get_ok('/story/16/01/28/0437240/.html')->status_is(301);
    $t->get_ok('/story/16/01/28/0437240.html')->status_is(301);
    $t->get_ok('/~torly/journal/599985/')->status_is(301);
    $t->get_ok('/~hogehoge/journal/599985/')->status_is(301);
    $t->get_ok('/~hogehoge/journal/0/')->status_is(404);

    $t->get_ok('/admin/css')->status_is(200);
    $t->get_ok('/submission/0/')->status_is(404);
    $t->get_ok('/submission/new')->status_is(200)->element_exists('#post-submission');

}


if ($t->app->mode eq 'test') {
    my $user = create_user($t->app, "test01", "foobar");
    ok($user, "create test user");
}

subtest 'login/logout test' => sub {
    $t->post_ok('/my/login' => {Accept => '*/*'} => form => {nickname => 'test01', passwd => 'foobar'})
      ->status_is(302);
    $t->get_ok('/')->element_exists('li#user-menu');
    $t->get_ok('/my/logout')->status_is(302);
    $t->get_ok('/')->element_exists_not('li#user-menu');
};

if ($t->app->mode eq 'test') {
    delete_user($t->app, "test01");
}

done_testing;
