# -*-Perl-*-
# Newslash::Model::Feeds related  tests
use Mojo::Base -strict;

use Test::More;
use Test::Mojo;
use Data::Dumper;
use DateTime::Format::MySQL;

my $t = Test::Mojo->new('Newslash::Web');

my $feeds = $t->app->model('feeds');
ok($feeds, "get Feeds instance");

subtest 'fetch feed' => sub {
    my $test_url = "https://mag.osdn.jp/rss";
    my $rss_feed = $feeds->fetch_url(\$test_url);
    ok($rss_feed, "fetch feed");
    ok($rss_feed->title, "feed title");
    ok($rss_feed->description, "feed description");
    ok($rss_feed->link, "feed url");
};

subtest 'create/select/update/delete feeds and items' => sub {
    plan skip_all => "mode is not 'test'" if ($t->app->mode ne 'test');

    my $test_title = "テストのFeed";
    my $test_url = "https://mag.osdn.jp/rss";
    my $test_name = "test_feed";

    my $rs = $feeds->select(url => $test_url);
    $feeds->delete(feed_id => $rs->{feed_id}) if $rs;

    my $feed_id = $feeds->create(url => $test_url,
                                 name => $test_name,
                                 title => $test_title);
    ok($feed_id, "create feed");

    my $feed = $feeds->select(feed_id => $feed_id);
    ok($feed, "select feed by feed_id");
    is($feed->{url}, $test_url, "retrive valid feed");

    $feed = $feeds->select(name => $test_name);
    ok($feed, "select feed by feed_name");
    is($feed->{url}, $test_url, "retrive valid feed");

    my $test_description = "説明";
    ok($feeds->update(feed_id => $feed_id,
                      description => $test_description),
       "update feed");

    $feed = $feeds->select(feed_id => $feed_id);
    is($feed->{description}, $test_description, "update feed description succeeded");

    $test_url = "https://ja.osdn.net/projects/tmdmaker/news/25544";
    $test_title = "TMD-Maker バージョン0.5.0 リリース - TMD-Maker(T字形ER図作成ツール)";
    my $test_content = << "EOT";
<div class="wiki-compact"><div class="indent">TMD-Maker(T字形ER図作成ツール)のバージョン0.5.0リリースしました。 </div><p>Javaアプリケーション(RCP)版とEclipse Plugin版の同時リリースです。 インストール方法や簡単な使用方法はリリースノート、ドキュメントをご覧ください。 </p><p>■ダウンロード </p><p><a href="http://osdn.jp/projects/tmdmaker/releases/">http://osdn.jp/projects/tmdmaker/releases/</a> </p><p>■ドキュメント </p><p><a href="http://tmdmaker.osdn.jp/doc/" class="external" rel="nofollow">http://tmdmaker.osdn.jp/doc/</a> </p><p><a href="http://osdn.jp/projects/tmdmaker/wiki">http://osdn.jp/projects/tmdmaker/wiki</a></p></div>
EOT
    my $test_content_type = "text/html";
    my $test_modified = DateTime->now;

    $rs = $feeds->items->select(url => $test_url);
    $feeds->items->delete(id => $rs->{id}) if $rs;

    my $item_id = $feeds->items->create(feed_id => $feed->{feed_id},
                                        url => $test_url,
                                        title => $test_title,
                                        content => $test_content,
                                        content_type => $test_content_type,
                                        modified => $test_modified);
    ok($item_id, "create item");
    my $item = $feeds->items->select(id => $item_id);
    ok($item, "select item");
    is($item->{url} => $test_url, "check url");
    is($item->{title} => $test_title, "check title");
    is($item->{content} => $test_content, "check content");
    is($item->{content_type} => $test_content_type, "check content");
    is($item->{modified} => DateTime::Format::MySQL->format_datetime($test_modified), "check content");

    $rs = $feeds->items->select(feed_id => $feed->{feed_id});
    ok($rs, "select item by feed_id");
    ok(@$rs, "retrive valid items");

    $rs = $feeds->items->select(feed_name => $test_name);
    ok($rs, "select item by feed_name");
    ok(@$rs, "retrive valid items");

    my $test_title2 = "ほげほげ";
    ok($feeds->items->update(id => $item_id,
                             title => $test_title2),
       "update item");
    $item = $feeds->items->select(id => $item_id);
    is($item->{title} => $test_title2, "update item title succeeded");

    ok($feeds->items->delete(id => $item_id), "delete item");
    ok($feeds->delete(feed_id => $feed_id), "delete feed");
};


# cleanup

done_testing();
