# -*-Perl-*-
# Newslash::Model::Polls tests
use Mojo::Base -strict;

use Test::More;
use Test::Mojo;
use Data::Dumper;

use Newslash::Util::TestMan;

my $t = Test::Mojo->new('Newslash::Web');

my $polls;
my ($admin, $user);
my $test_man = Newslash::Util::TestMan->new($t);

# prepare for test
if ($t->app->mode eq 'test') {
    $admin = $test_man->create_admin("polltestadmin");
    ok($admin, "create admin user");
    $user = $test_man->create_user("polltest");
    ok($user, "create test user");
}

subtest 'get polls object' => sub {
    $polls = $t->app->model('polls');
    ok($polls, 'get polls object');
};

subtest 'create/select/update/delete polls' => sub {
    plan skip_all => "mode is not 'test'" if ($t->app->mode ne 'test');

    # create question
    my $answers = ["item1", "item2", "item3"];
    my $params = {
                  question => "投票テスト",
                  tid => 736,
                  answers => $answers,
                 };
    my $qid = $polls->create($admin, $params);
    ok($qid, 'create poll');

    my $poll = $polls->select(question_id => $qid);
    ok($poll, 'select polls by qid');
    is($poll->{question}, $params->{question}, "valid question");
    is(@{$poll->{answers}}, @$answers, "check answers number");
    #warn(Dumper($poll));
    is($poll->{voters}, 0, "valid voters");
    my $index = 0;
    for my $answer (@$answers) {
        is($poll->{answers}->[$index]->{answer}, $answer, "check answer $answer");
        is($poll->{answers}->[$index]->{votes}, 0, "check count of answer $answer");
        $index++;
    }


    # vote
    ok(!$polls->vote($user, $qid, 0), "vote to invalid answer");
    ok($polls->vote($user, $qid, 1), "do vote");
    ok(!$polls->vote($user, $qid, 2), "duplicate vote not allowed");

    # select
    my $votes = $polls->voters->select(uid => $user->{uid}, qid => $qid);
    ok($votes, "voters->select");
    ok(@$votes, "vote accepted");

    $poll = $polls->select(qid => $qid);
    ok($poll, 'select polls by qid (2)');
    is($poll->{question}, $params->{question}, "valid question");
    is(@{$poll->{answers}}, @$answers, "check answers number");
    is($poll->{voters}, 1, "voters updated");
    for my $answer (@{$poll->{answers}}) {
        if ($answer->{aid} == 1) {
            is($answer->{votes}, 1, "votes updated $answer->{answer}");
        }
        else {
            is($answer->{votes}, 0, "votes not updated $answer->{answer}");
        }
    }

    ok($polls->hard_delete($qid), 'delete box');
    my $deleted_poll = $polls->select(qid => $qid);
    ok(!defined $deleted_poll, 'delete succeed');
};

$test_man->cleanup;

done_testing();
