# -*-Perl-*-
# Preprocessor plugin tests

use Mojo::Base -strict;
use Test::More;
use Test::Mojo;
use IO::Uncompress::Gunzip qw(gunzip);
use Newslash::Util::TestMan;
use Data::Dumper;
use Newslash::Eventd::Handlers;

my $t = Test::Mojo->new('Newslash::Web');

my $loader = $t->app->javascript_loader;
ok($loader, "get javascript_loader instance");

# check add_md5_to_path
is($loader->_add_md5_to_path("/css/newslash.css", "1234abcd"), "/css/newslash_1234abcd.css", "_add_md5_to_path ok");

# check strip_md5_from_path
is($loader->_strip_md5_from_path("/css/newslash_1234abcd.css"), "/css/newslash.css", "_strip_md5_from_path ok");

# get css/newslash.css
my $path = "/js/ns_all.js";

my $content = $loader->get_content($path);
ok($content, "get content");


if ($content) {
    ok($content->{content}, "get valid content");
    is($content->{type}, "application/javascript", "get valid type");
    is($content->{path}, $path, "get valid path");

    my $md5_path = $loader->get_md5_path($content->{path});
    is($md5_path, $loader->_add_md5_to_path($content->{path}, $content->{md5}), "get valid md5path");

    # expire all cache
    $loader->expire_cache($path, "all");
    my $content_nocache = $loader->get_content("$md5_path");

    # cached read
    my $content_cached = $loader->get_content("$md5_path");

    # expire inmemory cache
    $loader->expire_cache($path, "inmemory");
    my $content_kvs = $loader->get_content("$md5_path");

    ok($content_nocache, "nocache read succeed");
    ok($content_cached, "cached read succeed");
    ok($content_kvs, "kvscached read succeed");

    is_deeply($content_nocache, $content, "valid nocache read");
    is_deeply($content_cached, $content, "valid cached read");
    is_deeply($content_kvs, $content, "valid kvs cached read");

    # check gziped content
    my $c = $content_kvs->{gz_content};
    gunzip(\$c, \my $gunziped);
    ok(Encode::decode_utf8($gunziped) eq $content_kvs->{content}, "valid gzipped content");
}

done_testing();
