# -*-Perl-*-
# Preprocessor plugin tests

use Mojo::Base -strict;
use Test::More;
use Test::Mojo;
use IO::Uncompress::Gunzip qw(gunzip);
use Newslash::Util::TestMan;
use Data::Dumper;
use Newslash::Eventd::Handlers;

my $t = Test::Mojo->new('Newslash::Web');

my $pp = $t->app->preprocessor;
ok($pp, "get preprocessor instance");

# check add_md5_to_path
is($pp->_add_md5_to_path("/css/newslash.css", "1234abcd"), "/css/newslash_1234abcd.css", "_add_md5_to_path ok");

# check strip_md5_from_path
is($pp->_strip_md5_from_path("/css/newslash_1234abcd.css"), "/css/newslash.css", "_strip_md5_from_path ok");

# get css/newslash.css
my $path = "/css/newslash.css";

$pp->expire_cache($path, "all");
my $content = $pp->get_content($path);
ok($content, "get content");
ok(($content && $content->{content}), "get valid content");
ok(($content && $content->{type} eq "text/css"), "get valid type");

if ($content) {
    is($content->{path}, $path, "get valid path");
    my $md5_path = $pp->get_md5_path($path);
    is($md5_path, $pp->_add_md5_to_path($content->{path}, $content->{md5}), "get valid md5path");

    # check ggziped content
    my $gunzipped;
    gunzip(\$content->{gz_content}, \$gunzipped);
    ok(Encode::decode_utf8($gunzipped) eq $content->{content}, "valid gzipped content");

    # expire all cache
    $pp->expire_cache($md5_path, "all");
    my $content_nocache = $pp->get_content("$md5_path");
    ok($content_nocache, "nocache read succeed");

    # cached read
    my $content_cached = $pp->get_content("$md5_path");
    ok($content_cached, "cached read succeed");

    # expire inmemory cache
    $pp->expire_cache($md5_path, "inmemory");
    my $content_kvs = $pp->get_content("$md5_path");
    ok($content_kvs, "kvscached read succeed");

    # check gzipped content
    gunzip(\$content_nocache->{gz_content}, \$gunzipped);
    my $ret = ok(Encode::decode_utf8($gunzipped) eq $content->{content}, "valid gzipped content_nocache");
    #if ($ret) {
    #    is_deeply($content_nocache, $content, "valid nocache read");
    #}

    gunzip(\$content_cached->{gz_content}, \$gunzipped);
    $ret = ok(Encode::decode_utf8($gunzipped) eq $content->{content}, "valid gzipped content_cached");
    #if ($ret) {
    #    is_deeply($content_cached, $content, "valid cached read");
    #}

    gunzip(\$content_kvs->{gz_content}, \$gunzipped);
    $ret = ok(Encode::decode_utf8($gunzipped) eq $content->{content}, "valid gzipped content_kvs");
    #if ($ret) {
    #    is_deeply($content_kvs, $content, "valid kvs read");
    #}

}



done_testing();
