#!/usr/bin/perl
use strict;
use warnings;
use utf8;
use feature ':5.10';

use Getopt::Std;
use File::Basename;
use DBUtil;

my $PROGNAME = basename($0);
my $VERSION = '0.1.0';
my $table_sql_dir = "./sqls";
my $data_sql_dir = "./data";

sub main {
    my $opts = {};

    if (!getopts( "hvpcu:H:d:", $opts)) {
        usage('Options used incorrectly.');
    }

    if ($opts->{'h'}) {
        usage();
    }

    if (!$opts->{'u'}) { die "no user given! abort.\n" }
    if (!$opts->{'d'}) { die "no database given! abort.\n" }

    my $password = "";
    if ($opts->{'p'}) {
        system("stty -echo");
        while (!$password) {
            # ask password
            print "Password: ";
            chomp($password = <STDIN>);
            print "\n";
        }
        system("stty echo");
    }

    my $db = DBUtil->new(host => $opts->{'H'},
                         user => $opts->{'u'},
                         password => $password,
                         database => $opts->{'d'},
                         table_sql_directory => $table_sql_dir,
                         data_sql_directory => $data_sql_dir);

    if ($opts->{'c'}) {
        $db->delete_tables;
        return 0;
    }


    $db->initialize_database;
    return 0;
}

sub usage {
  print "*** $_[0] \n" if $_[0];
  print <<EOT;

Usage: $PROGNAME [OPTIONS]

This will create table in MySQL database.

Main options:
      -h    Help (this message)
      -v    Version
      -u    Username to use when connecting to server
      -H    Connect to host
      -p    Password to use when connecting to server
      -d    The database to use
EOT
  exit;
}

sub version {
  print <<EOT;


$PROGNAME $VERSION
EOT
  exit;
}

main();


